-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- info.lua

info = { ID = 0 }

-- print "[info]"

-- creation
function info:new (o)
	-- print "[info] new info"
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	-- what about gui text elements?
	x = 20
	y = 20
	width = 280
	height = 110

	texture_background_info = sim:getTexture( path .. "/resources/interface/frame_info.png" )
	texture_hp_bar = sim:getTexture( path .. "/resources/interface/hp_bar.png" )
	texture_sp_bar = sim:getTexture( path .. "/resources/interface/sp_bar.png" )
	texture_xp_bar = sim:getTexture( path .. "/resources/interface/xp_bar.png" )	
	
	info_window = sim:addWindow( 100, xcore.x2drect(x, y, x+width, y+height), "Selected Unit Info")
	info_window:setTitleBarVisible( false )

	image_background_specials = sim:addImage( 0, texture_background_info, xcore.x2dpoint(0, 0),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )
	image_background_specials:setParent( info_window )

	x = 82
	y = 23	
	image_hp_bar = sim:addImage( 0, texture_hp_bar, xcore.x2dpoint(x, y),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )
	image_hp_bar:setParent( info_window )

	y = 37
	image_sp_bar = sim:addImage( 0, texture_sp_bar, xcore.x2dpoint(x, y),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )
	image_sp_bar:setParent( info_window )

	x = 6
	y = 97	
	image_xp_bar = sim:addImage( 0, texture_xp_bar, xcore.x2dpoint(x, y),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )
	image_xp_bar:setParent( info_window )	
	-- Name - Class
	-- HP& SP bars
	-- 2x3 Stat Block
	-- Icon / level bar	
	
	-- thumbnail
	x = 10
	y = 24
	-- add upper accent
	texture_info_thumbnail = sim:getTexture( path .. "resources/interface/icon_.png" )
	image_info_thumbnail = sim:addImage( 0, texture_info_thumbnail, xcore.x2dpoint(x, y),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )
	image_info_thumbnail:setParent( info_window )

	-- text
	x = 10
	y = 2
	text_info_name = sim:addText( 0, "Unit Name", x, y, width, height)
	text_info_name:setParent( info_window )
	
	x = 108
	y = 60
	text_info_stats_1 = sim:addText( 0, "Atk 3\nDef 3\nMov 3", x, y, width, height)
	text_info_stats_1:setParent( info_window )

	x = 188
	y = 60
	text_info_stats_2 = sim:addText( 0, "Dam 3\nRng 3\nRes 3", x, y, width, height)
	text_info_stats_2:setParent( info_window )	

	x = 94
	y = 22
	text_info_hp_label = sim:addText( 0, "HP 30/30\nSP 8/8", x, y, width, height)
	text_info_hp_label:setParent( info_window )	

	x = 14
	y = 96
	text_info_xp_label = sim:addText( 0, "Lv. 2", x, y, width, height)
	text_info_xp_label:setParent( info_window )	
	text_info_xp_label:setFont( font )

	width = 68
	height = 20
	x = 10
	y = 76
	text_info_status = sim:addText( 0, "Ready", x, y, x+width, y+height)
	text_info_status:setParent( info_window )
	text_info_status:setAlignment( 2, 0 )
	self.view_mode = 1
	self.targetmob = -1
	self.displayedmob = -1
	self:hide()
	return o
end

function info:hide()
	-- print "[info] hide"
	info_window:setVisible( false )
end

function info:show()
	-- print "[info] show"
	info_window:setVisible( true )
	-- print "[info] /show"
end

function info:destroy()
	-- print "[info] destroy"
	info_window:destroy()
end

function info:clear()
	-- print("[info] clear")
	self.displayedmob = -1
	self.view_mode = 1
end

function info:update()
	-- print "[info] update unit info"
	info_text = ""
	if selected == nil and cursor_mob == nil then
		self:hide()
		return nil
	else
		self:show()
	end
	
	if cursor_mob ~= -1 then 
		self.targetmob = cursor_mob
	else
		self.targetmob = selected
	end
	
	-- print("selected : " .. selected )
	-- print("cursor_mob: " .. cursor_mob )
	-- print("targetmob = " .. self.targetmob )

	
	if self.targetmob > 0 and menu_dialog.visible == 0 then
		if self.displayedmob ~= self.targetmob then
			self.displayedmob = self.targetmob
			self.view_mode = 1
			self:updateData()
		end
	else
		self:hide()
	end		
	-- text_info:setText( info_text )
end

function info:updateData()	
	info_text = ""
	if game_map.mobs[ self.targetmob ].name == "none" or game_map.mobs[ self.targetmob ].name == game_map.mobs[ self.targetmob ].class_name then
		info_text = info_text .. game_map.mobs[ self.targetmob ].class_name 
	else
		info_text = info_text .. game_map.mobs[ self.targetmob ].name  .. " the " .. game_map.mobs[ self.targetmob ].class_name 
	end

	for n = 1, table.getn( game_map.mobs[ self.targetmob ].status_effects ) do
		info_text = info_text .. " - " .. game_map.mobs[ self.targetmob ].status_effects[ n ].effect_name
		if game_map.mobs[ self.targetmob ].status_effects[ n ].effect_duration > -1 then 
			info_text = info_text .. " (" .. game_map.mobs[ self.targetmob ].status_effects[ n ].effect_duration .. ")"
		end
	end

	text_info_name:setText( info_text )
	info_text = ""
	info_text = info_text .. "HP " .. game_map.mobs[ self.targetmob ].hp .. "/" .. game_map.mobs[ self.targetmob ]:getTotalHP() .."\n"
	info_text = info_text .. "SP " .. game_map.mobs[ self.targetmob ].sp .. "/" .. game_map.mobs[ self.targetmob ]:getTotalSP()
	
	-- set hp
	local damage = game_map.mobs[ self.targetmob ].hp 
	local hp = game_map.mobs[ self.targetmob ]:getTotalHP()
	local percent = ( ( hp-damage )/ hp ) * 188
	percent = 188-percent
	if percent <= 1 then percent = 1 end
	image_hp_bar:setSize( percent, 16 )
	
	-- set energy
	local drain = game_map.mobs[ self.targetmob ].sp
	local ep = game_map.mobs[ self.targetmob ]:getTotalSP()
	local percent = ( ( ep-drain )/ep) * 188
	percent = 188-percent
	if percent <= 1 then percent = 1 end
	image_sp_bar:setSize( percent, 16 )

	-- set xp
	local xp = game_map.mobs[ self.targetmob ].exp - game_map.mobs[ self.targetmob ]:getEXPToLevel( game_map.mobs[ self.targetmob ].level )
	local cap = game_map.mobs[ self.targetmob ]:getEXPToLevel( game_map.mobs[ self.targetmob ].level+1 ) - game_map.mobs[ self.targetmob ]:getEXPToLevel( game_map.mobs[ self.targetmob ].level )
	local percent = ( ( cap-xp )/cap) * 72
	percent = 72-percent
	if percent <= 1 then percent = 1 end
	image_xp_bar:setSize( percent, 16 )
	text_info_hp_label:setText( info_text )
	info_text = ""
	info_text = info_text .. " Lv." .. game_map.mobs[ self.targetmob ].level --  .. "(" .. game_map.mobs[ self.targetmob ].exp .."xp)\n"
	text_info_xp_label:setText( info_text )
	info_text = ""
	
	--[[
	self.attack  = 3
	self.defense = 3
	self.move    = 3
	self.range   = 1
	self.damage  = 1
	self.drain   = 1
	self.hp      = 1
	self.sp      = 3
	--]]

	if game_map.mobs[ self.targetmob ]:getTotalAttack() == game_map.mobs[ self.targetmob ].attack then
		info_text = info_text .. "Atk " .. game_map.mobs[ self.targetmob ].attack .."\n"
	else
		info_text = info_text 	.. "Atk " .. game_map.mobs[ self.targetmob ].attack .. "(" .. game_map.mobs[ self.targetmob ]:getTotalAttack()  .. ") \n"
	end

	if game_map.mobs[ self.targetmob ]:getTotalDefense() == game_map.mobs[ self.targetmob ].defense then
		info_text = info_text 	.. "Def " .. game_map.mobs[ self.targetmob ].defense .. "\n"
	else
		info_text = info_text 	.. "Def " .. game_map.mobs[ self.targetmob ].defense .. "(" .. game_map.mobs[ self.targetmob ]:getTotalDefense()  .. ") \n"
	end

	if game_map.mobs[ self.targetmob ]:getTotalMove() == game_map.mobs[ self.targetmob ].move then
		info_text = info_text 	.. "Mov " .. game_map.mobs[ self.targetmob ].move .. "\n"
	else
		info_text = info_text 	.. "Mov " .. game_map.mobs[ self.targetmob ].move .. "(" .. game_map.mobs[ self.targetmob ]:getTotalMove()  .. ")\n"
	end

	text_info_stats_1:setText( info_text )
	info_text = ""

	if game_map.mobs[ self.targetmob ]:getTotalDamage() == game_map.mobs[ self.targetmob ].damage then
		info_text = info_text 	.. "Dmg " .. game_map.mobs[ self.targetmob ].damage .. "\n"
	else
		info_text = info_text 	.. "Dmg " .. game_map.mobs[ self.targetmob ].damage .. "(" .. game_map.mobs[ self.targetmob ]:getTotalDamage()  .. ")\n"
	end

	if game_map.mobs[ self.targetmob ]:getTotalRange() == game_map.mobs[ self.targetmob ].range then
		info_text = info_text 	.. "Rng " .. game_map.mobs[ self.targetmob ].range .. "\n"
	else
		info_text = info_text 	.. "Rng " .. game_map.mobs[ self.targetmob ].range .. "(" .. game_map.mobs[ self.targetmob ]:getTotalRange()  .. ")\n"
	end

	if game_map.mobs[ self.targetmob ]:getTotalDrain() == game_map.mobs[ self.targetmob ].drain then
		info_text = info_text 	.. "Res " .. game_map.mobs[ self.targetmob ].drain .. "\n"
	else
		info_text = info_text 	.. "Res " .. game_map.mobs[ self.targetmob ].drain .. "(" .. game_map.mobs[ self.targetmob ]:getTotalDrain()  .. ")\n"
	end

	text_info_stats_2:setText( info_text )
	info_text = ""
	-- info_text = info_text .. "Skills | wpn:" .. game_map.mobs[ self.targetmob ].wpn_skill .." Arm:" .. game_map.mobs[ self.targetmob ].arm_skill .. "\n"

	if game_map.mobs[ self.targetmob ].wpn ~= nil then
		info_text = info_text .. "Wpn: " .. game_map.mobs[ self.targetmob ].wpn.item_name .. "\n"
	else
		if game_map.mobs[ self.targetmob ].type == 2 then
			info_text = info_text .. "Wpn: Claws \n"
		else
			info_text = info_text .. "Wpn: Fists \n"
		end
	end

	if game_map.mobs[ self.targetmob ].arm ~= nil then
		info_text = info_text .. "Arm: " .. game_map.mobs[ self.targetmob ].arm.item_name .. "\n"
	else
		if game_map.mobs[ self.targetmob ].type == 2 then
			info_text = info_text .. "Arm: None \n"
		else
			info_text = info_text .. "Arm: Clothes \n"
		end
	end

	if game_map.mobs[ self.targetmob ].acc ~= nil then
		info_text = info_text .. "Acc: " .. game_map.mobs[ self.targetmob ].acc.item_name .. "\n"
	else
		info_text = info_text .. "Acc: None \n"
	end		

	if self.view_mode == 2 then
		text_info_stats_1:setText( info_text )
		info_text = ""
		text_info_stats_2:setText( info_text )
	end

	info_text = ""
	-- info_text = info_text .. "\nSelected: " .. game_map.mobs[ self.targetmob ].name .."\n"
	-- 2 = moved, 3 = attacked
	
	info_text = info_text .. getMobState( self.targetmob )

	text_info_status:setText( info_text )
	info_text = ""

	image_info_thumbnail:setImage( sim:getTexture( path .. "resources/sprites/icon_" .. game_map.mobs[ self.targetmob ].graphic .. ".png" ) )

	-- text_info:setText( info_text )
end


function getMobState( mob_index )
	if game_map.mobs[ mob_index ].state == STATE_READY then
		return "Ready"
	elseif game_map.mobs[ mob_index ].state == STATE_ACTIVE then
		return "Active" -- can attack only
	elseif game_map.mobs[ mob_index ].state == STATE_DONE then
		return "Done" -- unit is done.
	elseif game_map.mobs[ mob_index ].state == STATE_RESERVE then
		return "Reserve" -- unit is in reserve	
	elseif game_map.mobs[ mob_index ].state == STATE_FLED then	
		return "Fled" -- unit has fled
	elseif game_map.mobs[ mob_index ].state == STATE_DOWN then	
		return "Down" -- unit is dead
	elseif game_map.mobs[ mob_index ].state == STATE_EXITED then	
		return "Exited" -- unit is dead		
	else
		return "Unknown"
	end
end

function getMobStatus( mob_index )
	return "Unknown"
end

-- end of file