-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- state_game.lua

state_game = state:new()

SUBSTATE_INTRO  = 0
SUBSTATE_INPUT  = 1
SUBSTATE_MOVE   = 2
SUBSTATE_ACT    = 3
SUBSTATE_EXIT   = 4
SUBSTATE_OVER   = 5
SUBSTATE_WAIT   = 6

-- creation
function state_game:new (o)
	-- print "[state_game] new state_game"
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	self.sid = 0

	-- add some text for debug info
	info_text = "Press esc to quit"

	-- add a skybox (top, bottom, left, right, front, back)
	-- sky = sim:addSkybox(500, path .. "resources/textures/sky.png",
	--  		path .. "resources/textures/sky.png",
	--  		path .. "resources/textures/sky.png",
	--   		path .. "resources/textures/sky.png",
	--  		path .. "resources/textures/sky.png",
	--  		path .. "resources/textures/sky.png")
	--  sky:setRotation( xcore.x3dpoint(0, 0, 0) )

	width = 128
	height = 64
	x = screen_width - 128 - 40
	y = 20
	texture_background_turn = sim:getTexture( path .. "/resources/interface/frame_128x64.png" )
	turn_window = sim:addWindow( 100, xcore.x2drect(x, y, x+width, y+height), "Turn Info")
	turn_window:setTitleBarVisible( false )
	turn_window:setVisible( false )

	image_background_specials = sim:addImage( 0, texture_background_turn, xcore.x2dpoint(0, 0), xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )
	image_background_specials:setParent( turn_window )
	
	x = 8
	y = 8
	text_game = sim:addText( 0, info_text, x, y, width-x, height-y)
	text_game:setAlignment( 2, 0 )
	text_game:setParent( turn_window )

	x = 8
	y = 8
	text_debug = sim:addText( 0, "" , x, y, 250-x, 500-y)
	
	
	x = 20
	y = screen_height - 40
	text_game_desc = sim:addText( 0, "text_game_desc", x, y, x+screen_width-20, y+40)
	text_game_desc:setVisible( false )
	-- text_game_desc:setAlignment(2, 0)

	tile_size  = 10

	-- create ui elements
	game_camera = camera:new()       -- add a static camera
	game_camera:reset()
	game_manager = manager:new()     -- add control surface
	
	game_history = history:new()     -- add combat log
	game_alert = alert:new()         -- exit yes / no dialog

	menu_inventory = inventory:new() -- equipment list
	menu_base = base:new()           -- user's base
	menu_actions = actions:new()     -- actions menu
	menu_specials = specials:new()   -- actions menu	
	menu_info = info:new()           -- unit info
	menu_support = support:new()     -- support menu

	menu_summary = summary:new()     -- level summary
	menu_unitlist = unitlist:new()   -- unit list
	menu_dialog = dialog:new()       -- conversation system

	-- mouse inputs
	lclickx  = 0
	rclickx  = 0
	rlclickx = 0
	rclicky  = 0
	gui_event = 0
	
	-- set animation frames
	ani_len    = 9
	ani_face   = 4
	ani_long = {}
	for x = 0, ani_len-1 do
		ani_long[ x+1 ] = {}
		for y = 0, ani_face-1 do
			-- print( "walk animation (x,y)=(".. x .."," .. y ..")" )
			ani_long[ x+1 ][ y+1 ] = {}
			ani_long[ x+1 ][ y+1 ] = xcore.x2dpoint( x*(1/ani_len), y*(1/ani_face) )
		end
	end

	ani_len    = 6	
	ani_short = {}
	for x = 0, ani_len-1 do
		ani_short[ x+1 ] = {}
		for y = 0, ani_face-1 do
			-- print( "death animation (x,y)=(".. x .."," .. y ..")" )
			ani_short[ x+1 ][ y+1 ] = {}
			ani_short[ x+1 ][ y+1 ] = xcore.x2dpoint( x*(1/ani_len), y*(1/ani_face) )
		end
	end
	m = xcore.xmatrix4()
	
	show_debug = 0
	score_total = 0
	dungeon_level = 0
	target_index = -1
	return o
end

-- called on state_game enter
function state_game:enter ()
	-- print "[state_game] enter state_game"

	snd:stopSource( sound_menu_theme )
	
	self.time = 0
	turn_window:setVisible( true )
	text_game_desc:setVisible( true )
	game_history:show()
	game_history:clear()
	game_manager:show()

	dungeon_level = 0
	-- map_depth = 12
	-- map_size  = 12

	map_nodes = 1
	map_droprate = 90 -- 90
	map_feature_count = 0

	score_total = 0
	wait = 0
	delay = 100

	-- create (or load) objects and map
	local_user = user:new()	-- user data
	local_user:load()
	game_map = map:new()

	effects = {}
	self:startLevel()
end

-- called on state_game exit
function state_game:exit()
	-- print "[state_game] exit state_game"

	-- hide text
	turn_window:setVisible( false )
	text_game_desc:setVisible( false )
	game_history:hide()
	game_alert:hide()
	game_manager:hide()

	menu_inventory:hide()
	menu_base:hide()
	menu_info:hide()	
	menu_actions:hide()
	menu_specials:hide()
	menu_support:hide()
	menu_unitlist:hide()
	menu_dialog:hide()

	-- destroy map
	game_map:destroy()

	local_user:save()
	local_user:destroy()
	game_map = nil
	local_user = nil

	for i=n,1,-1 do
		if effects[i] ~= nil then
			effects[i]:destroy()
			-- effects[i] = nil
			table.remove ( effects, i )
		end    
	end
	-- print "[state_game] /exit"
end

function state_game:hideMenus()
	-- print ( "[state_game] hideMenus" )
	game_history:hide()
	game_alert:hide()
	game_manager:hide()

	menu_inventory:hide()
	menu_base:hide()
	menu_info:hide()	
	menu_actions:hide()
	menu_specials:hide()
	menu_support:hide()
	menu_unitlist:hide()
	menu_dialog:hide()
end

-- called to advance state_game
function state_game:input ()
	-- print ( "[state_game] input" )
	
	-- update events
	r = self:doevents()

	game_camera:update()
	game_manager:update()
	game_map:update()

	-- draw everything
	sim:update()
	self:updateData()

	for i = 1, table.getn( game_map.mobs ) do
		game_map.mobs[ i ]:setAniFace()
		game_map.mobs[ i ]:updateAnimation()
	end

	n = table.getn( effects )
	for i=n,1,-1 do
		if effects[i]:update() == 0 then
			effects[i]:destroy()
			-- effects[i] = nil
			table.remove ( effects, i )
		end
	end

	info_text = "\nLocation (" .. cursor_map_tile.X .."," .. cursor_map_tile.Y ..") : "
	if cursor_mob ~= -1 then
		if game_map.mobs[ cursor_mob ].name == "none" or game_map.mobs[ cursor_mob ].name == game_map.mobs[ cursor_mob ].class_name then
			info_text = info_text .. "You see a " .. game_map.mobs[ cursor_mob ].class_name .."\n"
		else
			info_text = info_text .. "You see " .. game_map.mobs[ cursor_mob ].name .."\n"
			
		end
	else
		-- info_text = info_text .. "\n You see terrain \n"
		-- print( "cursor tile: " .. cursor_map_tile.X .."," .. cursor_map_tile.Y  )
		if cursor_map_tile.X > 0 and cursor_map_tile.X <= map_size and cursor_map_tile.Y > 0 and cursor_map_tile.Y <= map_size then
			a = getNodeIndexAtGrid( cursor_map_tile )
			if a ~= nil then
					info_text = info_text .. "You see a " .. game_map.nodes[ a ].name  .. "\n"
			elseif game_map.tiles ~= nil then
				info_text = info_text .. "You see ".. game_map.tiles[ cursor_map_tile.X ][ cursor_map_tile.Y].text .. ".\n"
			end
		else
			info_text = info_text .. "You see nothing.\n"
		end
	end
	text_game_desc:setText( info_text )	
	menu_info:update()


	-- print("update sound...")
	snd:setListenerPosition( 0, 0, 0 )
	snd:setSourcePosition( sound_attack, 0, 0, 0) -- xcore.x3dpoint( 0, 0, 0 )
	snd:setSourcePosition( sound_cast, 0, 0, 0)   -- xcore.x3dpoint( 0, 0, 0 )
	snd:setSourcePosition( sound_hit, 0, 0, 0)   -- xcore.x3dpoint( 0, 0, 0 )
	snd:setSourcePosition( sound_loot, 0, 0, 0)   -- xcore.x3dpoint( 0, 0, 0 )
	snd:setSourcePosition( sound_win, 0, 0, 0)   -- xcore.x3dpoint( 0, 0, 0 )
	snd:setSourcePosition( sound_lose, 0, 0, 0)   -- xcore.x3dpoint( 0, 0, 0 )	
	snd:update()

	if substate == SUBSTATE_INTRO then -- start dialog
		if menu_dialog.visible == 0 then
			-- self:hideMenus()
			menu_dialog:show()
		else
			menu_dialog:input()
		end
		
	elseif substate == SUBSTATE_INPUT then -- collect input
		if phase == 1 then
			if lclickx > 0 and gui_event == 0 then
				-- print("[state_game] left click")
				game_map:clearHighlights()
				menu_info:clear()
				if ready_action == 0 then
					-- select a mob or tile
					setSelected( cursor_mob )
					if cursor_mob == -1 then
						-- check for stairs, and show base panel menu if so
						if cursor_map_tile.X > 0 and cursor_map_tile.X <= map_size and cursor_map_tile.Y > 0 and cursor_map_tile.Y <= map_size then
							if game_map.tiles[ cursor_map_tile.X ][ cursor_map_tile.Y].tile_type == 2 then
								-- show list of players off-map assets
								menu_base:show()
							end
						end
					end
					cam_target = xcore.x2dpoint( cursor_map_tile.X, cursor_map_tile.Y )
				else
					-- select the target of an action
					if ready_action == 1 then
						if game_map.tiles[ cursor_map_tile.X ][ cursor_map_tile.Y].valid == 1 then
							-- print( "[state_game] attack target at grid " .. cursor_map_tile.X .. ", " .. cursor_map_tile.Y )
							target_index = getMobIndexAtGrid( cursor_map_tile )
							if target_index ~= nil then
								self:attack( selected, target_index )
								game_map.mobs[ selected ]:setAnimation( ANIMATION_TYPE_ATTACK )
							end
						else
							-- print( "[state_game] target is not valid" )
						end
					elseif ready_action == 2 then
						if game_map.tiles[ cursor_map_tile.X ][ cursor_map_tile.Y ].valid == 1 then
							self:setMove( selected, cursor_map_tile )
						else
							print( "[state_game] target position is not valid" )
						end
					elseif ready_action == 4 then
						-- print( "[state_game] target special" )
						if game_map.tiles[ cursor_map_tile.X ][ cursor_map_tile.Y].valid == 1 then
							-- print( "[state_game] use special on target at grid " .. cursor_map_tile.X .. ", " .. cursor_map_tile.Y )
							target_index = getMobIndexAtGrid( cursor_map_tile )
							if target_index ~= nil then
								self:useSpecial( selected, target_index )
							end
						else
							-- print( "[state_game] target is not valid" )
						end
					else
						-- print( "[state_game] unknown action" )
					end
					ready_action = 0
				end
			end
			if rclickx > 0 and gui_event == 0 then
				-- print("[state_game] right click (cancel)")
				if selected ~= -1 then 
					setSelected( -1 )
					ready_action = 0
					game_map:clearHighlights()
				elseif selected == -1 then
					-- print ("[state_game] show support menu")
					menu_base:hide()
					menu_support:show()
					game_alert:hide()
					menu_unitlist:hide()
				end
			end

			lclickx = 0
			rclickx = 0
			gui_event = 0

			-- user turn
			if self:userTurn() == -1 then
				print("[state_game] wtf, return ENUM_SCORES !?")
				return ENUM_SCORES
			end
		elseif phase == 2 then
			-- if gamemode == 1 then
				-- ai turn
				self:cpuTurn()
			-- else
				-- remote player turn
			-- end
		end
	elseif substate == SUBSTATE_MOVE then -- animate movement
		-- print("SUBSTATE_MOVE")
		if selected == -1 then selected = cpu_selected end
		self:moveMob( selected )

	elseif substate == SUBSTATE_ACT then -- animate attack / special
		-- count to 20
		wait = wait + 1
		-- print("wait:" .. wait )
		-- print("delay:" .. delay )
		if wait > delay then
			wait = 0
			substate = SUBSTATE_INPUT
		end

	elseif substate == SUBSTATE_EXIT then -- end dialog
		if menu_dialog.visible == 0 then
			-- self:hideMenus()
			menu_dialog:show()
		else
			menu_dialog:input()
		end

	elseif substate == SUBSTATE_OVER then -- game over
		self:endLevel()
		substate = SUBSTATE_WAIT
	elseif substate == SUBSTATE_WAIT then -- wait?	
		-- waiting...
	else
		print("[state_game] Error : undefined substate")
	end
	return r
end

-- yup. 'buckets of d6' style wargame ftw.
function rollDice( n )
	local total = 0
	for i = 1, n do
		total = total + math.random(1, 6)
	end
	return total
end

function state_game:moveMob( n )
	-- print "\n[state_game] moveMob"
	if game_map.mobs[ n ] == nil then print ("error!") return nil end 
	char_dest = game_map.mobs[ n ].dest
	char_tile = game_map.mobs[ n ].grid

	if game_map.mobs[ n ].step > 0 then
		-- print ("[state_game] step: " .. game_map.mobs[ n ].step )
		pos = getTilePosByID( pathList[ game_map.mobs[ n ].step ].tileID )
		local step_pixel = game_map.tiles[ pos.X ][ pos.Y ].position 
		local char_pixel = game_map.mobs[ n ].position 
		if  step_pixel.X ~= char_pixel.X or step_pixel.Z ~= char_pixel.Z then
			speed = 0.04
			if char_pixel.X > step_pixel.X then
				char_pixel.X = char_pixel.X - speed
				game_map.mobs[ n ]:setFacing( FACING_TYPE_EAST )
			elseif char_pixel.X < step_pixel.X then
				char_pixel.X = char_pixel.X + speed
				game_map.mobs[ n ]:setFacing( FACING_TYPE_WEST )
			elseif char_pixel.Z > step_pixel.Z then
				char_pixel.Z = char_pixel.Z - speed
				game_map.mobs[ n ]:setFacing( FACING_TYPE_NORTH )
			elseif char_pixel.Z < step_pixel.Z then
				char_pixel.Z = char_pixel.Z + speed
				game_map.mobs[ n ]:setFacing( FACING_TYPE_SOUTH )
			end
			
			-- print ( "[state_game] update char co-ords" )
			char_tile = getMapTileFromPoint( xcore.x2dpoint ( char_pixel.X, char_pixel.Z ) )

			game_map.mobs[ n ]:setPosition( char_pixel )
			game_map.mobs[ n ].grid.X = char_tile.X
			game_map.mobs[ n ].grid.Y = char_tile.Y

			if math.floor( char_pixel.X ) == math.floor( step_pixel.X ) then
				if math.floor( char_pixel.Z ) == math.floor( step_pixel.Z ) then
					-- hmm, this is ugly
					char_pixel.X = math.floor( char_pixel.X )
					char_pixel.Z = math.floor( char_pixel.Z )
					game_map.mobs[ n ]:setPosition( char_pixel )

					-- TODO: don't go on to next step if there is a character in the way
					if game_map.mobs[ n ].step > 1 then
						nextpos = getTilePosByID( pathList[ game_map.mobs[ n ].step-1 ].tileID )
						if nextpos ~= nil then
							-- print ("[state_game] step : " .. game_map.mobs[ n ].step  )
							-- print ("[state_game] next pos : " .. nextpos.X .. "," .. nextpos.Y  )
							mobAtGrid = getMobIndexAtGrid( xcore.x2dpoint( nextpos.X , nextpos.Y ) )
							if mobAtGrid ~= nil then
								-- print("[state_game] next step in path is obstructed by a character")
								game_map.mobs[ n ].step = 0
							end
						end
					end

					game_map.mobs[ n ].step = game_map.mobs[ n ].step-1
					game_map.mobs[ n ].has_moved = game_map.mobs[ n ].has_moved + 1
					if game_map.mobs[ n ].has_moved >= game_map.mobs[ n ]:getTotalMove() then
						game_map.mobs[ n ].step = 0
					end

					if game_map.mobs[ n ].step <= 0 then
						-- print "[state_game] destination reached, end walk animation"
						-- pickup any items
						local a = getNodeIndexAtGrid( game_map.mobs[ n ].grid )
						if a ~= nil then self:pickup( n ) end
						-- check for exit
						self:checkExit( n )
						substate = SUBSTATE_INPUT
						-- print("[state_game] (crash debug) n :" .. n )
						game_map.mobs[ n ].has_moved = 0
						game_map.mobs[ n ]:setAnimation( ANIMATION_TYPE_IDLE )
					end
				end
			end
		else
			-- print "[state_game] error : improper alignment"
		end
	else
		-- print "[state_game] no step, end walk cycle"
		-- pickup any items
		local a = getNodeIndexAtGrid( game_map.mobs[ n ].grid )
		if a ~= nil then self:pickup( n ) end
		-- check for exit
		self:checkExit( n )
		substate = SUBSTATE_INPUT
		-- print("[state_game] (crash debug) n :" .. n )
		game_map.mobs[ n ].has_moved = 0
		game_map.mobs[ n ]:setAnimation( ANIMATION_TYPE_IDLE )
	end
	-- print "[state_game] /moveMob"
end

function state_game:checkExit( n ) -- mob_index, node_index
	-- print("[state_game] checkExit :" .. n)
	-- print("[state_game] loc :" .. game_map.mobs[ n ].grid.X .. "" .. game_map.mobs[ n ].grid.Y )
	if game_map.mobs[ n ].state ~= STATE_EXITED then
		if game_map.tiles[ game_map.mobs[ n ].grid.X ][ game_map.mobs[ n ].grid.Y ].tile_type == TILE_TYPE_START then
			if game_map.mobs[ n ].owner_id == 1 then
				-- print "[state_game] allied mob at start zone, set as fled"
				game_history:addItem( game_map.mobs[ n ]:getName() .. " has retreated."  )
				game_map.mobs[ n ]:setState( STATE_EXITED )
				game_map.mobs[ n ]:setGrid( -10000, -10000 )
				setSelected( -1 )
			else
				-- print "[state_game] enemy mob at start zone, set as escaped"
				game_history:addItem( game_map.mobs[ n ]:getName() .. " has escaped."  )
				game_map.mobs[ n ]:setState( STATE_EXITED )
				game_map.mobs[ n ]:setGrid( -10000, -10000 )
				setSelected( -1 )
			end
			return true
		elseif game_map.tiles[ game_map.mobs[ n ].grid.X ][ game_map.mobs[ n ].grid.Y ].tile_type == TILE_TYPE_EXIT then
			if game_map.mobs[ n ].owner_id == 1 then
				-- print "[state_game] allied mob at end zone, set as escaped / end level"
				game_history:addItem( game_map.mobs[ n ]:getName() .. " has escaped." )
				-- self:endLevel() -- not sure if i should call this...
				self:showEndDialog()
			else
				-- print "[state_game] enemy mob at end zone, do nothing..."
			end
			return true
		end
	end
	return false
end

function state_game:pickup( a ) -- mob_index, node_index
	-- print("[state_game] pickup")
	if game_map.mobs[ a ].owner_id ~= 1 then print("[state_game]  mobs cant pickup yet.") return nil end
	
	b = getNodeIndexAtGrid( game_map.mobs[ a ].grid )
	-- print( game_map.mobs[ a ].name .. " has picked up " .. game_map.nodes[ b ].name )
	if game_map.nodes[ b ].gold > 0 then
		game_history:addItem( "You have aqquired " .. game_map.nodes[ b ].gold .. " gold." ) -- todo: variable currency name
		local_user.gold = local_user.gold + game_map.nodes[ b ].gold
	else
		for i = 1, table.getn( game_map.nodes[ b ].items ) do
			-- print("[state_game] add item " .. game_map.nodes[ b ].items[i] .. " to local user's inventory" )
			menu_inventory:addItem( game_map.nodes[ b ].items[i] )
			table.remove ( game_map.nodes[ b ].items, i )
		end
	end
	-- get some exp for collecting loot
	game_map.mobs[ a ]:addExp( 1 )

	-- play sound effect
	snd:playSource( sound_loot )
	
	game_map.nodes[ b ]:destroy()
	table.remove ( game_map.nodes, b )
end

function state_game:useSpecial( a, b )
	print( "\n[state_game] useSpecial" )
	if current_ability == -1 then return nil end
	if game_map.mobs[ a ].special_abilities[ current_ability ].stat_drain > game_map.mobs[ a ].sp then
		game_history:addItem ( game_map.mobs[ a ]:getName() .. " does not have enough SP to use the special ability.")
		return nil
	end
	if game_map.mobs[ a ]:canSpecial() == false then
		game_history:addItem ( game_map.mobs[ a ]:getName() .. " is prevented from using the special ability.")
		return nil
	end
	
	if game_map.mobs[ a ].special_abilities[ current_ability ].stat_damage > 0 then
		print("[state_game] direct damage ability")
		damage = rollDice( game_map.mobs[ a ].special_abilities[ current_ability ].stat_damage )
		print( "[state_game] ability_radius:" .. game_map.mobs[ a ].special_abilities[ current_ability ].ability_radius )
		-- test against everyone within radius
		for i = 1, table.getn( game_map.mobs ) do
			dist = getDistance( game_map.mobs[ b ].grid, game_map.mobs[ i ].grid )
			if  dist <= game_map.mobs[ a ].special_abilities[ current_ability ].ability_radius then
				game_history:addItem( game_map.mobs[ i ]:getName() .. " is hit for " .. game_map.mobs[ a ].special_abilities[ current_ability ].stat_damage .. "d6 = " .. damage .. " damage." )
				-- print damage effect
				if game_map.mobs[ i ]:takeDamage( damage ) == true then
					game_map.mobs[ a ]:addExp( game_map.mobs[ i ]:getLevel()+1 )
				else
					game_map.mobs[ a ]:addExp( 1 )
				end
			end
		end
	elseif game_map.mobs[ a ].special_abilities[ current_ability ].stat_damage < 0 then
		print("[state_game] healing ability")
		damage = rollDice( math.abs( game_map.mobs[ a ].special_abilities[ current_ability ].stat_damage ) )
		game_history:addItem( game_map.mobs[ b ]:getName() .. " is healed for " .. math.abs( game_map.mobs[ a ].special_abilities[ current_ability ].stat_damage ) .. "d6 = " ..  math.abs(damage) .. " HP." )
		-- print damage effect
		p = game_map.mobs[ b ].position
		i = table.getn( effects ) + 1
		effects[ i ] = visual:new()
		effects[ i ]:setPosition( xcore.x3dpoint( p.X, p.Y+5, p.Z ) )
		effects[ i ]:setType( VISTAL_TYPE_TEXT )
		effects[ i ]:setColour( 255, 0, 255, 0 )
		effects[ i ]:setText( "+" .. math.abs(damage) )
		game_map.mobs[ b ].hp = game_map.mobs[ b ].hp + damage
		if game_map.mobs[ b ].hp >= game_map.mobs[ b ]:getTotalHP() then
			game_map.mobs[ b ].hp = game_map.mobs[ b ]:getTotalHP()
		end
		game_map.mobs[ a ]:addExp( 1 )
	end
	if game_map.mobs[ a ].special_abilities[ current_ability ].ability_effect > 0 then
		print("[state_game] status effect type " .. game_map.mobs[ a ].special_abilities[ current_ability ].effect_type )
		if game_map.mobs[ a ].special_abilities[ current_ability ].effect_type == EFFECT_TYPE_CLEAR then
			print("[state_game] status clear")
			n = table.getn( game_map.mobs[ b ].status_effects )
			for i = n, 1, -1 do
				-- only get rid of negitive status effects ( EFFECT_TYPE_STRIP should get rid of positive ones )
				if game_map.mobs[ b ].status_effects[i].effect_type ~= EFFECT_TYPE_BUFF and game_map.mobs[ b ].status_effects[i].effect_type ~= EFFECT_TYPE_SUMMON then
					table.remove ( game_map.mobs[ b ].status_effects, i )
				end
			end
		elseif game_map.mobs[ a ].special_abilities[ current_ability ].effect_type == EFFECT_TYPE_SUMMON then
			print("[state_game] summon ability")
			-- hmmm....
		else
			print("[state_game] status effect ability type")
			-- first check to see if ability stacks
			-- game_map.mobs[ a ].special_abilities[ current_ability ].effect_name
			local stack = 0
			local apply = 1
			if game_map.mobs[ a ].special_abilities[ current_ability ].effect_stack == 1 then
				stack = 1
			end

			for n = 1, table.getn( game_map.mobs[ b ].status_effects ) do
				if game_map.mobs[ a ].special_abilities[ current_ability ].effect_name == game_map.mobs[ b ].status_effects[ n ].effect_name then
					apply = 0
					if stack == 1 then
						-- the ability stacks; increase duration
						game_map.mobs[ b ].status_effects[ n ].effect_duration = game_map.mobs[ b ].status_effects[ n ].effect_duration + game_map.mobs[ a ].special_abilities[ current_ability ].effect_duration
					else
						-- ability does not stack, refresh duration
						game_map.mobs[ b ].status_effects[ n ].effect_duration = game_map.mobs[ a ].special_abilities[ current_ability ].effect_duration
					end
					
				end
			end
			if apply == 1 then
				-- apply the effect on mob b
				n = table.getn( game_map.mobs[ b ].status_effects ) + 1
				game_map.mobs[ b ].status_effects[ n ] = {}
				game_map.mobs[ b ].status_effects[ n ].effect_name = game_map.mobs[ a ].special_abilities[ current_ability ].effect_name
				game_map.mobs[ b ].status_effects[ n ].effect_type = game_map.mobs[ a ].special_abilities[ current_ability ].effect_type
				game_map.mobs[ b ].status_effects[ n ].effect_duration = game_map.mobs[ a ].special_abilities[ current_ability ].effect_duration
				game_map.mobs[ b ].status_effects[ n ].effect_magnitude = game_map.mobs[ a ].special_abilities[ current_ability ].effect_magnitude
				game_map.mobs[ b ].status_effects[ n ].effect_break = game_map.mobs[ a ].special_abilities[ current_ability ].effect_break
				game_map.mobs[ b ].status_effects[ n ].effect_stack = game_map.mobs[ a ].special_abilities[ current_ability ].effect_stack
				game_map.mobs[ b ].status_effects[ n ].effect_icon = game_map.mobs[ a ].special_abilities[ current_ability ].effect_icon
				game_map.mobs[ b ].status_effects[ n ].effect_animation = game_map.mobs[ a ].special_abilities[ current_ability ].effect_animation
				game_map.mobs[ b ].status_effects[ n ].effect_attack = game_map.mobs[ a ].special_abilities[ current_ability ].effect_attack
				game_map.mobs[ b ].status_effects[ n ].effect_defense = game_map.mobs[ a ].special_abilities[ current_ability ].effect_defense
				game_map.mobs[ b ].status_effects[ n ].effect_move = game_map.mobs[ a ].special_abilities[ current_ability ].effect_move
				game_map.mobs[ b ].status_effects[ n ].effect_range = game_map.mobs[ a ].special_abilities[ current_ability ].effect_range
				game_map.mobs[ b ].status_effects[ n ].effect_damage = game_map.mobs[ a ].special_abilities[ current_ability ].effect_damage
				game_map.mobs[ b ].status_effects[ n ].effect_drain = game_map.mobs[ a ].special_abilities[ current_ability ].effect_drain
				game_map.mobs[ b ].status_effects[ n ].effect_sp = game_map.mobs[ a ].special_abilities[ current_ability ].effect_sp
				game_map.mobs[ b ].status_effects[ n ].effect_hp = game_map.mobs[ a ].special_abilities[ current_ability ].effect_hp
			end
		end
	else
		print("[state_game] unknown effect id " .. game_map.mobs[ a ].special_abilities[ current_ability ].ability_effect )
	end

	if b ~= nil then
		if game_map.mobs[ a ].position.X > game_map.mobs[ b ].position.X then
			game_map.mobs[ a ]:setFacing( FACING_TYPE_EAST )
		elseif game_map.mobs[ a ].position.X < game_map.mobs[ b ].position.X then
			game_map.mobs[ a ]:setFacing( FACING_TYPE_WEST )
		elseif game_map.mobs[ a ].position.Z > game_map.mobs[ b ].position.Z then
			game_map.mobs[ a ]:setFacing( FACING_TYPE_NORTH )
		elseif game_map.mobs[ a ].position.Z < game_map.mobs[ b ].position.Z then
			game_map.mobs[ a ]:setFacing( FACING_TYPE_SOUTH )
		end

		game_history:addItem ( game_map.mobs[ a ]:getName() .. " uses " .. game_map.mobs[ a ].special_abilities[ current_ability ].ability_name .. " on " .. game_map.mobs[ b ]:getName() )
	else
		game_history:addItem ( game_map.mobs[ a ]:getName() .. " uses " .. game_map.mobs[ a ].special_abilities[ current_ability ].ability_name )
	end
	
	-- print( "[state_game] ability drain : " .. game_map.mobs[ a ].special_abilities[ current_ability ].stat_drain )
	-- print( "[state_game] user drain : " .. game_map.mobs[ a ]:getTotalDrain() )
	game_map.mobs[ a ].sp = game_map.mobs[ a ].sp - game_map.mobs[ a ].special_abilities[ current_ability ].stat_drain

	if game_map.mobs[ selected ].state == STATE_ACTIVE then
		game_map.mobs[ selected ]:setState( STATE_DONE )
	else
		game_map.mobs[ selected ]:setState( STATE_ACTIVE )
		activations = activations - 1
		target_index = -1
	end
	substate = SUBSTATE_ACT
	menu_info:update()

	-- show special graphic todo : fixme.
	-- print( "[state_game] special ability_name : " .. game_map.mobs[ a ].special_abilities[ current_ability ].ability_name )
	-- print( "[state_game] special ability_graphic : " .. game_map.mobs[ a ].special_abilities[ current_ability ].ability_graphic )
	if game_map.mobs[ a ].special_abilities[ current_ability ].ability_graphic == "effect_physical" then
		game_map.mobs[ a ]:setAnimation( ANIMATION_TYPE_ATTACK )
		delay = 100
		snd:playSource( sound_hit ) -- play sound effect		
	elseif game_map.mobs[ a ].special_abilities[ current_ability ].ability_graphic == "effect_ice_arrow" then
		n = table.getn( effects ) + 1
		effects[ n ] = visual:new()
		effects[ n ]:setPosition( game_map.mobs[ a ].position )
		effects[ n ]:setEndPoint( game_map.mobs[ b ].position )
		effects[ n ]:setType( VISUAL_TYPE_ICE )
		game_map.mobs[ a ]:setAnimation( ANIMATION_TYPE_SPECIAL )
		delay = 100
		snd:playSource( sound_cast ) -- play sound effect		
	elseif game_map.mobs[ a ].special_abilities[ current_ability ].ability_graphic == "effect_fire_ball" then
		n = table.getn( effects ) + 1
		effects[ n ] = visual:new()
		effects[ n ]:setPosition( game_map.mobs[ b ].position )		
		effects[ n ]:setType( VISUAL_TYPE_FIRE )
		game_map.mobs[ a ]:setAnimation( ANIMATION_TYPE_SPECIAL )		
		delay = 100
		snd:playSource( sound_cast ) -- play sound effect		
	elseif game_map.mobs[ a ].special_abilities[ current_ability ].ability_graphic == "effect_heal" then
		n = table.getn( effects ) + 1
		effects[ n ] = visual:new()
		effects[ n ]:setPosition( game_map.mobs[ b ].position )		
		effects[ n ]:setType( VISUAL_TYPE_LIGHT )
		game_map.mobs[ a ]:setAnimation( ANIMATION_TYPE_SPECIAL )
		delay = 100
		snd:playSource( sound_cast ) -- play sound effect		
	elseif game_map.mobs[ a ].special_abilities[ current_ability ].ability_graphic == "effect_smite" then
		n = table.getn( effects ) + 1
		effects[ n ] = visual:new()
		effects[ n ]:setPosition( game_map.mobs[ b ].position )				
		effects[ n ]:setType( VISUAL_TYPE_DARK )
		game_map.mobs[ a ]:setAnimation( ANIMATION_TYPE_SPECIAL )
		delay = 100
		snd:playSource( sound_cast ) -- play sound effect
	else
		-- print("[state_game] unknown effect")
		game_map.mobs[ a ]:setAnimation( ANIMATION_TYPE_SPECIAL )
	end
	menu_info:update()
end

function state_game:attack( a, b )
	-- print( "\n[state_game] attack" )
	if game_map.mobs[ a ]:canAttack() == false then
		game_history:addItem ( game_map.mobs[ a ]:getName() .. " is prevented from attacking.")
		return nil
	end

	-- face target?
	if game_map.mobs[ a ].position.X > game_map.mobs[ b ].position.X then
		game_map.mobs[ a ]:setFacing( FACING_TYPE_EAST )
	elseif game_map.mobs[ a ].position.X < game_map.mobs[ b ].position.X then
		game_map.mobs[ a ]:setFacing( FACING_TYPE_WEST )
	elseif game_map.mobs[ a ].position.Z > game_map.mobs[ b ].position.Z then
		game_map.mobs[ a ]:setFacing( FACING_TYPE_NORTH )
	elseif game_map.mobs[ a ].position.Z < game_map.mobs[ b ].position.Z then
		game_map.mobs[ a ]:setFacing( FACING_TYPE_SOUTH )
	end
	substate = SUBSTATE_INPUT
	delay = 100
	game_history:addItem( game_map.mobs[ a ]:getName() .. " has attacked " .. game_map.mobs[ b ]:getName() .. "." )
	
	-- check to see if mob is outnumbered and if so apply an overwelm attack bonus
	-- print( "getAdjacentHostileCount: " .. game_map.mobs[ b ]:getAdjacentHostileCount() )
	bonus_dice = game_map.mobs[ b ]:getAdjacentHostileCount()-1
	-- roll dice!
	attack_roll = rollDice( game_map.mobs[ a ]:getTotalAttack()+bonus_dice ) + game_map.mobs[ a ].level
	defense_roll =  rollDice( game_map.mobs[ b ]:getTotalDefense() ) + game_map.mobs[ b ].level

	if bonus_dice <= 0 then
		bonus_dice = 0
		game_history:addItem ("Attack roll " .. game_map.mobs[ a ]:getTotalAttack() .. "d6: " .. attack_roll .. " v.s. Defense roll " .. game_map.mobs[ b ]:getTotalDefense() .. "d6: " .. defense_roll .. " ")
	else
		game_history:addItem( game_map.mobs[ b ]:getName() .. " is outnumbered!" )
		game_history:addItem ("Attack roll " .. game_map.mobs[ a ]:getTotalAttack() .. "d6+" .. bonus_dice .. ": " .. attack_roll .. " v.s. Defense roll " .. game_map.mobs[ b ]:getTotalDefense() .. "d6: " .. defense_roll .. " ")
	end

	if attack_roll >= defense_roll then

		damage = rollDice( game_map.mobs[ a ]:getTotalDamage() )
		if attack_roll >= defense_roll *2 then
			damage = damage*2
			game_history:addItem( "Critical Hit! " .. game_map.mobs[ a ]:getName() .. " hit " .. game_map.mobs[ b ]:getName() .. " for " .. game_map.mobs[ a ]:getTotalDamage() .. "d6*2 = " .. damage .. " critical damage!" )
		else
			game_history:addItem( game_map.mobs[ a ]:getName() .. " hit " .. game_map.mobs[ b ]:getName() .. " for " .. game_map.mobs[ a ]:getTotalDamage() .. "d6 = " .. damage .. " damage!" )
		end

		if game_map.mobs[ b ]:takeDamage( damage ) == true then
			game_map.mobs[ a ]:addExp( game_map.mobs[ b ]:getLevel()+1 )
		else
			game_map.mobs[ a ]:addExp( 1 )
		end
		-- play sound effect
		snd:playSource( sound_hit )
	else
		game_history:addItem( "The attack missed." )
		-- todo : print *miss* effect
		p = game_map.mobs[ b ].position
		i = table.getn( effects ) + 1
		effects[ i ] = visual:new()
		effects[ i ]:setPosition( xcore.x3dpoint( p.X, p.Y+5, p.Z ) )
		effects[ i ]:setType( VISTAL_TYPE_TEXT )
		effects[ i ]:setText( "*MISS*" )
		
		if attack_roll == game_map.mobs[ a ]:getTotalAttack() then
			-- todo : this.
			-- print("[state_game] critical failure / weapon break")
			-- game_map.mobs[ a ].wep:destroy()
			-- game_map.mobs[ a ].wep = nil
		end
		
		-- play sound effect
		snd:playSource( sound_attack )
	end

	if game_map.mobs[ selected ].state == STATE_ACTIVE then
		game_map.mobs[ selected ]:setState( STATE_DONE )
	else
		game_map.mobs[ selected ]:setState( STATE_ACTIVE )
		activations = activations - 1
		target_index = -1
	end
	substate = SUBSTATE_ACT
	menu_info:update()
end

function state_game:setMove( a, pos )
	-- print( "[state_game] setMove" )
	-- print( " action : move; mob : " .. game_map.mobs[ a ]:getName() .. "; position " .. pos.X .. ", " .. pos.Y )
	if game_map.mobs[ a ]:canMove() == false then
		game_history:addItem ( game_map.mobs[ a ]:getName() .. " is prevented from moving.")
		return nil
	end
	
	game_history:addItem( game_map.mobs[ a ]:getName() .. " moves to position " .. pos.X .. ", " .. pos.Y )
	game_map:clearLabels()
	-- findPath( cam_target, pos )
	game_map.mobs[ a ].step = findPath( game_map.mobs[ a ].grid, pos ) 
	-- game_map:drawPath()
	
	-- game_map.mobs[ a ]:setGrid(  pos.X, pos.Y  )
	game_map.mobs[ a ]:setDest( pos.X, pos.Y  )
	substate = SUBSTATE_MOVE
	game_map.mobs[ a ]:setAnimation( ANIMATION_TYPE_WALK )
	
	if game_map.mobs[ a ].state == STATE_ACTIVE then
		game_map.mobs[ a ]:setState( STATE_DONE )
	else
		game_map.mobs[ a ]:setState( STATE_ACTIVE )
		activations = activations - 1
		target_index = -1
	end
	menu_info:update()
end

function state_game:highlightSpecial()
	-- print( "[state_game] highlightSpecial" )
	if game_map.mobs[ selected ].state == STATE_READY or game_map.mobs[ selected ].state == STATE_ACTIVE then
		if substate == SUBSTATE_INPUT then
			current_ability = listbox_specials:getSelected()+1
			if game_map.mobs[ selected ].special_abilities[ current_ability ].ability_type == 2 then
				-- highlight enemies
				-- print("[state_game] highlight enemies")
				game_map:highlightEnemies( selected, current_ability )
				ready_action = 4
			else
				-- highlight allies
				-- print("[state_game] highlight allies")
				game_map:highlightAllies( selected, current_ability )
				ready_action = 4
			end
		end
	end
end

function state_game:userTurn()
	-- print "[state_game] userTurn"
	if allies_in_progress + allies_ready == 0  or allies_activated >= current_round * allied_activations then
		-- user's turn is over
		self:endPhase()
	elseif enemies_defeated == enemies then
		-- print "[state_game] floor cleared."
		self:showEndDialog()
		-- self:endLevel()
	elseif allies_defeated == allies then
		self:endLevel()
		-- show loss menu / highscore / exit to title screen
		return -1
	end
	return 1
end

function state_game:getScore()
	-- print("[state_game] getScore" )
	local p = 0
	-- loop over mob array, counting the enemy mobs killed
	for n = 1, table.getn( game_map.mobs ) do
		if game_map.mobs[ n ].owner_id ~= 1 then
			if game_map.mobs[ n ].state == STATE_DOWN then
				p = p + game_map.mobs[ n ]:getLevel()
			end
		end
	end
	return p
end

function state_game:startLevel()
	print "[state_game] startLevel"
	turn_window:setVisible( true )
	text_game_desc:setVisible( true )
	dungeon_level = dungeon_level + 1
	menu_summary:hide()
	game_history:show()
	game_manager:show()
	game_history:clear()
	game_history:addItem( "You have descended to level " .. dungeon_level )

	-- reset data
	turn               = 0 -- turn ends when every mob has taken an action
	phase              = 1 -- what player is active
	rounds             = 0 -- number of rounds this turn
	current_round      = 0 -- number of rounds this turn
	activations        = 1 -- number of activations per round	
	enemy_activations  = 0 -- number of enemy activations remaining
	allied_activations = 0 -- number of activations remaining
	selected           = -1
	target_index       = -1
	ready_action       = 0
	substate = SUBSTATE_INTRO

	map_nodes = math.random( 0, map_sequence )
	map_droprate = 99 - ( dungeon_level * map_sequence )

	-- print("node range : 0 - " ..  map_sequence )
	-- print("map_nodes :" .. map_nodes )
	-- print("map_droprate :" .. map_droprate )

	game_map:spawnUser()
	game_map:create()
	game_map:populate()
	menu_dialog:load( map_sequence, dungeon_level )
	self:changeTurn()
end

function state_game:showEndDialog() -- stupid name?
	-- print "[state_game] showEndDialog"
	-- menu_dialog:load( map_sequence, dungeon_level )	
	game_alert:hide()

	menu_inventory:hide()
	menu_base:hide()
	menu_actions:hide()
	menu_specials:hide()	
	menu_info:hide()
	menu_support:hide()

	menu_summary:hide()
	menu_unitlist:hide()
	
	substate = SUBSTATE_EXIT
	menu_dialog.step = 100
	menu_dialog:update()
end

function state_game:endLevel()
	-- print "[state_game] endLevel"
	-- calc level score
	if allies_defeated + allies_exited == allies and allies_exited ~= 0 then
		-- print("[state_game] user withdrawn")
		bonus = 0
		score = 0
	elseif allies_defeated == allies then
		-- print("[state_game] user defeated")
		bonus = 0
		score = 0
	else
		-- print("[state_game] user victorious")
		bonus = rollDice( dungeon_level * map_sequence )
		if bonus < 0 then bonus = 0 end
		score = self:getScore()

		local_user.gold = local_user.gold + bonus
		
		-- bonus exp 
		-- print( "Clear exp : " .. math.floor(score / allies) )
		for n = 1, table.getn( game_map.mobs ) do
			if game_map.mobs[ n ].owner_id == 1 and game_map.mobs[ n ].state ~= STATE_DOWN then
				game_map.mobs[ n ]:addExp( math.floor( score / allies ) )
			end
		end
		if dungeon_level == map_depth and local_user.sequence == map_sequence then
			-- print("[state_game] user has cleared sequenced stage")
			local_user.sequence = local_user.sequence + 1
			local_user.gold = local_user.gold + map_reward
		end
		local_user:save()		
	end
	
	game_map:resetMobs()
	game_map:saveParty()
	game_map:saveInventory()	
	game_map:clear()
	phase = -1
	selected = -1
	-- show summery
	turn_window:setVisible( false )
	text_game_desc:setVisible( false )
	game_manager:hide()
	game_history:hide()
	menu_actions:hide()
	menu_specials:hide()
	menu_info:hide()
	menu_summary:show()
end

function state_game:endPhase()
	-- print "[state_game] endPhase"
	-- game_history:addItem( "The phase has ended." )
	menu_base:hide()
	menu_actions:hide()
	menu_specials:hide()
	menu_info:hide()
	menu_info:clear()

	-- change phase
	phase = phase + 1
	target_index = -1
	if phase > 2 then -- player_count
		self:changeRound()
	end
end

function state_game:changeRound()
	-- print "[state_game] changeRound"
	-- game_history:addItem( "The round has ended." )
	phase = 1
	current_round = current_round + 1
	target_index = -1

	if allies_defeated + allies_exited == allies then
		-- print("do retreat")
		dungeon_level = 0
		map_depth = 0
		setSelected( -1 )
		self:endLevel()
	elseif current_round > rounds then
		self:changeTurn()
	end	
end

function state_game:changeTurn()
	-- print "[state_game] changeTurn"
	if turn ~= 0 then
		game_history:addItem( "Turn " .. turn .. " has ended." )
	end

	-- reset activated mobs, natural healing, expire buffs, etc...
	for n = 1, table.getn( game_map.mobs ) do
		game_map.mobs[ n ]:updateStatus()
		if game_map.mobs[ n ].state == STATE_DONE then
			game_map.mobs[ n ]:setState( STATE_READY )
		elseif game_map.mobs[ n ].state == STATE_EXITED then		
			if game_map.mobs[ n ].owner_id == 1 then
				game_map.mobs[ n ]:setState( STATE_RESERVE )
			end
		end
	end
	self:updateData()
	menu_actions:hide()
	menu_specials:hide()
	menu_info:hide()
	menu_info:clear()
	menu_inventory:hide()

	turn = turn + 1
	phase = 1
	current_round = 1
	target_index = -1
	rounds = 0

	allied_activations = allies_ready / enemies_ready
	enemy_activations = enemies_ready / allies_ready

	if allied_activations < 1 then allied_activations = 1 end
	if enemy_activations < 1 then enemy_activations = 1 end	

	rounds = allies_ready / allied_activations

	game_history:addItem( "Turn " .. turn .. " has started." )
end

function state_game:updateData()
	allies             = 0 -- number of allied units total
	allies_activated   = 0 -- number of activated units
	allies_ready       = 0 -- number of ready units
	allies_in_progress = 0 -- number of units currently being activated
	allies_defeated    = 0 -- number of units defeated or retreated
	allies_exited      = 0 -- number of units who went down

	enemies            = 0 -- number of enemy mobs total
	enemies_activated  = 0 -- number of activated enemy units
	enemies_ready      = 0 -- number of ready enemy units
	enemies_in_progress= 0 -- number of units currently being activated
	enemies_defeated   = 0 -- number of units defeated or retreated
	enemies_exited     = 0 -- number of units defeated or retreated	
	
	-- update data
	for i = 1, table.getn( game_map.mobs ) do
		if game_map.mobs[ i ].owner_id == 1 then -- user_id?
			allies = allies + 1
			if game_map.mobs[ i ].state == STATE_READY or game_map.mobs[ i ].state == STATE_RESERVE then
				allies_ready = allies_ready + 1
			elseif game_map.mobs[ i ].state == STATE_ACTIVE then
				allies_in_progress = allies_in_progress + 1
			elseif game_map.mobs[ i ].state == STATE_DONE then
				allies_activated = allies_activated + 1
			elseif game_map.mobs[ i ].state == STATE_EXITED then
				allies_exited = allies_exited + 1
			else
				allies_defeated = allies_defeated + 1
			end
		else
			enemies = enemies + 1
			if game_map.mobs[ i ].state == STATE_READY or game_map.mobs[ i ].state == STATE_RESERVE then
				enemies_ready = enemies_ready + 1
			elseif game_map.mobs[ i ].state == STATE_ACTIVE then
				enemies_in_progress = enemies_in_progress + 1
			elseif game_map.mobs[ i ].state == STATE_DONE then
				enemies_activated = enemies_activated + 1
			elseif game_map.mobs[ i ].state == STATE_EXITED then
				enemies_exited = enemies_exited + 1
			else
				enemies_defeated = enemies_defeated + 1
			end
		end
	end
end

function state_game:cpuTurn()
	-- print "\n[state_game] cpuTurn"

 	if enemies_in_progress + enemies_ready == 0 or enemies_activated >= current_round * enemy_activations then
		-- user's turn is over
		-- print "[state_game] end cpu turn"
		self:endPhase()
	else
		-- print "[state_game] cpu has units to activate"
		-- select a unit
		if selected == -1 then
			-- print "[state_game] find new unit"
			-- test a random unit, if is enemy and has not acted yet, flag it as activated.
			random_unit = math.random(1, table.getn( game_map.mobs ) )
			if game_map.mobs[ random_unit ].state == STATE_READY and game_map.mobs[ random_unit ].owner_id ~= 1 then
				-- print ( "[state_game] activating : " .. game_map.mobs[ random_unit ].name  )
				game_history:addItem( game_map.mobs[ random_unit ]:getName() .. " is now active." )
				-- set unit as selected
				selected = random_unit
				cpu_selected = selected
			end
		else -- unit is selected!
			-- check to see units state
			-- print "[state_game] unit is selected"
			if game_map.mobs[ selected ].state == STATE_READY or game_map.mobs[ selected ].state == STATE_ACTIVE then
				if game_map.mobs[ selected ].state == STATE_READY then
					-- print("[state_game] mob state : STATE_READY ")
				else
					-- print("[state_game] mob state : STATE_ACTIVE ")
				end
				-- check for a useable special ability
				local use_special = 0
				current_ability = game_map.mobs[ selected ]:getRandomSpecial()
				if current_ability ~= nil then
					if game_map.mobs[ selected ].special_abilities[ current_ability ].stat_drain <= game_map.mobs[ selected ].sp then
						-- has enough MP to use it.
						-- has one? check to see if it is used on ally or enemy
						if game_map.mobs[ selected ].special_abilities[ current_ability ].ability_type == 2 then
							-- used on enemies
							game_map.mobs[ selected ]:targetClosestHostile()
							if game_map.mobs[ selected ].target_id ~= -1 then
								-- print("[state_game] ability : " .. game_map.mobs[ selected ].special_abilities[ current_ability ].ability_name )
								-- print("[state_game] target_id : " .. game_map.mobs[ selected ].target_id )
								-- print("[state_game] range_to_target : " .. game_map.mobs[ selected ].range_to_target )
								-- print("[state_game] ability range : " .. game_map.mobs[ selected ].special_abilities[ current_ability ].stat_range )
								-- check to see if there is a target in range
								if game_map.mobs[ selected ].range_to_target <= game_map.mobs[ selected ].special_abilities[ current_ability ].stat_range then
									-- print("[state_game] hostile mob in range of offensive special, activate it")
									use_special = 1
								end
							end
						else
							-- used on allies
							game_map.mobs[ selected ]:targetClosestAlly()
							if game_map.mobs[ selected ].target_id ~= -1 then
								-- print("[state_game] ability : " .. game_map.mobs[ selected ].special_abilities[ current_ability ].ability_name )
								-- print("[state_game] target_id : " .. game_map.mobs[ selected ].target_id )
								-- print("[state_game] range_to_target : " .. game_map.mobs[ selected ].range_to_target )
								-- print("[state_game] ability range : " .. game_map.mobs[ selected ].special_abilities[ current_ability ].stat_range )
								-- check to see if there is a target in range
								if game_map.mobs[ selected ].range_to_target <= game_map.mobs[ selected ].special_abilities[ current_ability ].stat_range then
									-- print("[state_game] allied mob in range of support special, activate it")
									-- should only activate support specials on mobs below max HP.
									use_special = 1
								end
							end
						end
					end
				else
					-- no special
				end

				if math.random(1, 100) > 50 then use_special = 0 end -- 50% chance to not use special...
				if game_map.mobs[ selected ]:canSpecial() == false then use_special = 0 end
				
				if use_special == 1 then
					-- print("[state_game] use special")
					target_index = getMobByMobID( game_map.mobs[ selected ].target_id )
					if target_index ~= nil then
						self:useSpecial( selected, target_index )
					end
				else
					game_map.mobs[ selected ]:targetClosestHostile()
					if game_map.mobs[ selected ].target_id == -1 then
						-- print("[state_game] no target, waiting")
						game_map.mobs[ selected ].state = STATE_DONE
					end

					-- if no specials avaliable then check for attacks
					if game_map.mobs[ selected ].range_to_target <= game_map.mobs[ selected ]:getTotalRange() and game_map.mobs[ selected ]:canAttack() == true then
						-- print("[state_game] can attack")
						-- print("range_to_target:" .. game_map.mobs[ selected ].range_to_target )
						-- print("range:" .. game_map.mobs[ selected ].range )
						
						target_index = getMobByMobID( game_map.mobs[ selected ].target_id )
						if target_index ~= nil then
							self:attack( selected, target_index )
							game_map.mobs[ selected ]:setAnimation( ANIMATION_TYPE_ATTACK )
						end

					else
						-- print("[state_game] no attack, move instead")
						-- if no attacks chose a move loc instead
						if game_map.mobs[ selected ]:getTotalMove() > 0 and game_map.mobs[ selected ]:canMove() == true then
							p = nil
							target_index = getMobByMobID( game_map.mobs[ selected ].target_id )
							if target_index ~= nil then
								p = game_map.mobs[ target_index ].grid
							end
							if p ~= nil then
								self:setMove( selected, p )
								
								--[[ 
								print( "[state_game]" .. game_map.mobs[ selected ].name .. " moves towards target " .. game_map.mobs[ target_index ].name )
								range = findPath( game_map.mobs[ selected ].grid, p )
								-- print("[state_game] range = " .. range )
								if range ~= -1 then
									-- move towards target loc
									print( "[state_game] target in range, move" )
									-- for i = 1, table.getn( pathList ) do
										-- print("[" .. i .. "]:" .. pathList[i].tileID .. " - " .. parent )
									-- end
									-- print( "[state_game] can move to " ..  table.getn( pathList ) - game_map.mobs[ selected ].move )	
									game_map.mobs[ selected ].step = range
									game_map:clearLabels()
									-- game_map:drawPath()
									game_map.mobs[ selected ]:setDest( p.X, p.Y )
									
									if game_map.mobs[ selected ].state == STATE_READY then
										game_map.mobs[ selected ].state = STATE_ACTIVE
									else
										game_map.mobs[ selected ].state = STATE_DONE
									end
									substate = SUBSTATE_MOVE
									game_map.mobs[ selected ]:setAnimation( ANIMATION_TYPE_WALK )
								else
									print("[state_game] path obstructed, waiting...")
									game_map.mobs[ selected ].state = STATE_DONE
									enemy_activations = enemy_activations - 1
								end
								-- ]]

								-- TODO: select a random square adjacent to the target
							end
						else
							-- move to random adjacent tile
							-- fuck it, dont move.
							-- print("[state_game] no path to target, waiting")
							-- if no attacks just end turn
							game_map.mobs[ selected ]:setState( STATE_DONE )
							enemy_activations = enemy_activations - 1
						end
						-- if set to flee or ranged, then move away from target
					end
				end
			else
				-- print("[state_game] unit not in ready state : " .. game_map.mobs[ selected ].state )
			end
			if game_map.mobs[ selected ].state == STATE_DONE then
				-- print("[state_game] unit finished")
				-- unit has done everything, so end turn
				game_history:addItem( game_map.mobs[ selected ]:getName() .. " waits." )				
				self:checkExit( selected )
				selected = -1
			end
		end
		-- AI stances?
	end
end

-- called when changing state_game
function state_game:transition ( i ) -- transition to state_game v
	-- print "[state_game] transition"
end

-- This is a custom function to take care of events.
function state_game:doevents()
	-- print "[state_game] doevents"
	lp = 0
	while lp == 0 do            -- infinite loop, continues while still messeges in que.
		e = sim:getEvent()  -- get oldest event out ov que.
		if e == nil then
			lp = 1
		else
			-- print ("event type :" .. e:getEventType() )
			if e:getEventType() == xcore.XMOUSE_EVENT then
				-- print ("mouse event type " .. e:getMouseEventType() )
				if e:getMouseEventType() == xcore.XMOUSE_MOVED then
					mousex = e:getMouseX()
					mousey = e:getMouseY()
				end
				if e:getMouseEventType() == xcore.XMOUSE_L_LEFT_UP then
					-- print ( "[state_game] lmb click - activate item at index " .. selected_item_index )
					lclickx = e:getMouseX()
					lclicky = e:getMouseY()
				end
				if e:getMouseEventType() == xcore.XMOUSE_R_PRESSED_DOWN then
					rclickx = e:getMouseX()
					rclicky = e:getMouseY()
				end
				if e:getMouseEventType() == xcore.XMOUSE_R_PRESSED_DOWN then
					game_camera.rotate = 1
					game_camera.center_x = mousex
					game_camera.center_y = mousey
				end
				if e:getMouseEventType() == xcore.XMOUSE_R_LEFT_UP then
					game_camera.rotate = 0
				end
				if e:getMouseEventType() == xcore.XMOUSE_WHEEL then
					game_camera.radius = game_camera.radius + (e:getMouseWheel()*5)

				end
			elseif e:getEventType() == xcore.XGUI_EVENT then
				-- print("gui event type "  .. e:getGUIEventType()  )
				gui_event = 1
			end
			sim:removeEvent( )
		end
	end
	if sim:isGUI( "button_deploy_pressed" ) then menu_base:deploy() end
	if sim:isGUI( "button_attack_pressed" ) then self:showAttack() end
	if sim:isGUI( "button_move_pressed" ) then self:showMove() end
	if sim:isGUI( "button_special_pressed" ) then self:showSpecials() end
	if sim:isGUI( "button_specials_use_pressed" ) then self:highlightSpecial() end
	if sim:isGUI( "button_item_pressed" ) then self:showItems() end
	if sim:isGUI( "button_view_pressed" ) then
		if menu_info.view_mode == 1 then menu_info.view_mode = 2 else menu_info.view_mode = 1 end
		menu_info:updateData()
	end
	if sim:isGUI( "button_wait_pressed" ) then self:doWait() end
	if sim:isGUI( "button_support_list_pressed" ) then
		menu_unitlist:show()
		game_alert:hide()
	end
	if sim:isGUI( "button_support_end_pressed" ) then self:endTurn() end
	if sim:isGUI( "button_support_exit_pressed" ) then
		if phase == 1 and substate == SUBSTATE_INPUT then
			if selected == -1 then
				game_alert:show()
				menu_base:hide()
				menu_unitlist:hide()
			else
				setSelected( -1 )
				menu_base:hide()
				menu_actions:hide()
				menu_info:hide()
				menu_support:hide()
				menu_inventory:hide()
				menu_specials:hide()
				menu_unitlist:hide()
			end
			game_map:clearHighlights()
			ready_action = 0
		end
	end	
	if sim:isGUI( "button_alert_accept_pressed" ) then
		dungeon_level = 0
		return ENUM_MENU
	end
	if sim:isGUI( "button_alert_cancel_pressed" ) then game_alert:hide() end
	if sim:isGUI( "button_summary_accept_pressed" ) then
		if dungeon_level < map_depth then
			self:startLevel()
		else
			menu_summary:hide()			
			return ENUM_STRATEGY
		end
	end
	if sim:isGUI( "button_inventory_use_pressed" ) then menu_inventory:useItem() end
	if sim:isGUI( "table_unitlist_select" ) then menu_unitlist:selectUnit() end
	if sim:isKeyUp( "key_pressed_debug" ) then
		show_debug = show_debug + 1
		if show_debug > 4 then show_debug = 0 end
	end
	if sim:isKey( "key_pressed_cam_up" ) then game_camera.alpha = game_camera.alpha - 0.001* cam_speed end
	if sim:isKey( "key_pressed_cam_down" ) then game_camera.alpha = game_camera.alpha + 0.001* cam_speed end
	if sim:isKey( "key_pressed_cam_left" ) then game_camera.theta = game_camera.theta - 0.001* cam_speed end
	if sim:isKey( "key_pressed_cam_right" ) then game_camera.theta = game_camera.theta + 0.001* cam_speed end

	if sim:isKeyDown( "key_pressed_cam_rotate" ) then
		-- print("is down key_pressed_cam_rotate")
		game_camera.rotate = 1
		game_camera.center_x = mousex
		game_camera.center_y = mousey
	end
	if sim:isKeyUp( "key_pressed_cam_rotate" ) then
		-- print("is up key_pressed_cam_rotate")
		game_camera.rotate = 0
	end
	if sim:isKey( "key_pressed_cam_out" ) then
		game_camera.radius = game_camera.radius + 5
		if game_camera.radius > 1000 then game_camera.radius = 1000 end
	end
	if sim:isKey( "key_pressed_cam_in" ) then
		game_camera.radius = game_camera.radius - 5
		if game_camera.radius < 5 then game_camera.radius = 5 end
	end

	if sim:isKeyUp( "key_pressed_info_mode" ) then
		if menu_info.view_mode == 1 then menu_info.view_mode = 2 else menu_info.view_mode = 1 end
		menu_info:updateData()
	end

	if sim:isKey( "key_pressed_exit_state" ) then
		if phase == 1 and substate == SUBSTATE_INPUT then
			if selected == -1 then
				game_alert:show()
				menu_base:hide()
			else
				selected = -1
				menu_base:hide()
				menu_actions:hide()
				menu_info:hide()
				menu_support:hide()
				menu_inventory:hide()
				menu_specials:hide()
				menu_unitlist:hide()
			end
			game_map:clearHighlights()
			ready_action = 0
		end
	end

	-- hotkeys only work on player's turn
	if phase == 1 and substate == SUBSTATE_INPUT then
		if sim:isKeyUp( "key_pressed_attack" ) then 
			self:showAttack()
		end
		if sim:isKeyUp( "key_pressed_move" ) then 
			self:showMove()
		end
		if sim:isKeyUp( "key_pressed_wait" ) then 
			self:doWait()
		end
		if sim:isKeyUp( "key_pressed_end_turn" ) then 
			self:endTurn()
		end
		if sim:isKeyUp( "key_pressed_home" ) then
			local pos = game_map:getStart()
			cam_target = xcore.x2dpoint( pos.X, pos.Y )
			if getMobIndexAtGrid( pos ) ~= nil then
				setSelected( getMobIndexAtGrid( pos ) )
			else
				setSelected( -1 )
				hmenu_base:show()			
			end
		end
		if sim:isKeyUp( "key_pressed_special" ) then 
			self:showSpecials()
		end
	end

	-- return ENUM_MENU else
	return ENUM_GAME
end

function state_game:showAttack ()
	-- print "[state_game] showAttack"
	if selected == -1 then return end
	if game_map.mobs[ selected ].state == STATE_READY or game_map.mobs[ selected ].state == STATE_ACTIVE then
		if substate == SUBSTATE_INPUT then
			game_map:highlightAttack( selected )
			menu_inventory:hide()
			menu_specials:hide()
			ready_action = 1
		end
	end
end

function state_game:showMove ()
	-- print "[state_game] showMove"
	if selected == -1 then return end
	if game_map.mobs[ selected ].state == STATE_READY or game_map.mobs[ selected ].state == STATE_ACTIVE then
		if substate == SUBSTATE_INPUT then
			ready_action = 2
			game_map:highlightMovement( selected )
			menu_inventory:hide()
			menu_specials:hide()
		end
	end
end

function state_game:showSpecials ()
	-- print "[state_game] showSpecials"
	if selected == -1 then return end
	if substate == SUBSTATE_INPUT then
		game_map:clearHighlights()
		menu_inventory:hide()
		menu_specials:show()
		-- menu_specials:show()
		ready_action = 4
	end
end

function state_game:showItems ()
	-- print "[state_game] showItems"
	if selected == -1 then return end
	if substate == SUBSTATE_INPUT then
		ready_action = 5
		menu_inventory:show()
		menu_specials:hide()
		game_map:clearHighlights()
	end
end

function state_game:doWait ()
	-- print "[state_game] doWait"
	if substate == SUBSTATE_INPUT then		
		menu_base:hide()
		menu_actions:hide()
		menu_specials:hide()
		menu_info:hide()
		menu_inventory:hide()
		game_map:clearHighlights()
		
		if selected == -1 then return end
		if self:checkExit( selected ) == false then
			game_map.mobs[ selected ]:setState( STATE_DONE )
			game_history:addItem( game_map.mobs[ selected ]:getName() .. " waits." )
		end
		
		target_index = -1
		selected = -1
		ready_action = 0
		current_ability = -1
	end
end

function state_game:endTurn ()
	-- end all alied mob turns
	for n = 1, table.getn( game_map.mobs ) do
		if game_map.mobs[ n ].owner_id == 1 then
			-- print("unit : " .. game_map.mobs[ n ].name )
			if game_map.mobs[ n ].state == STATE_READY or game_map.mobs[ n ].state == STATE_ACTIVE then
				game_map.mobs[ n ]:setState( STATE_DONE )
				-- print("unit deployed, set state done")
			elseif game_map.mobs[ n ].state == STATE_RESERVE then
				game_map.mobs[ n ]:setState( STATE_EXITED )
				-- print("unit in reserve, set state exited")
			end
		end
	end			
	menu_unitlist:hide()
	menu_support:hide()
end

function setSelected( i )
	-- print "[state_game] setSelected"
	selected = i
	-- hide everything
	menu_base:hide()
	menu_actions:hide()
	menu_specials:hide()
	menu_info:hide()
	menu_inventory:hide()
	game_alert:hide()
	menu_support:hide()
	menu_unitlist:hide()

	if selected > 0 then
		menu_info.view_mode = 1
		menu_info:show()
		if game_map.mobs[ selected ].owner_id == 1 then -- user_id?
			if allies_in_progress > 0 then
				if game_map.mobs[ selected ].state == STATE_ACTIVE then
					menu_actions:show()
				else
					menu_actions:hide()
				end
			else
				if game_map.mobs[ selected ].state == STATE_READY then
					menu_actions:show()
				end
			end
		end
	end
end

-- eof