-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- visual.lua

VISUAL_TYPE_NONE        = 0
VISUAL_TYPE_PHYSICAL    = 1
VISUAL_TYPE_FIRE        = 2
VISUAL_TYPE_ICE         = 3
VISUAL_TYPE_LIGHT       = 4
VISUAL_TYPE_DARK        = 5

visual = { ID = 0 }

-- creation
function visual:new (o)
	-- print "[visual] new visual"
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	o.id       = 0
	o.life     = 0
	o.time     = 0
	o.endpoint = nil
	o.position = xcore.x3dpoint (0, 0, 0)

	o.colour = xcore.xcolour( 255, 255, 255, 255 )
	self.text = "powerup"
	self.parent_id = -1
	self.fade = 255
	return o
end

function visual:update ()
	-- print "[visual] update"
	-- count down life
	self.time = self.time + 1
	-- print ( "self.time:" .. self.time )
	-- print ( "self.life:" .. self.life )
	if self.life ~= -1 then
		if self.time >= self.life*2 then
			-- print "[visual] remove"
			return 0
		elseif self.time > (self.life/2) then
			-- print "[visual] stop emitter"
			if self.field ~= nil then 
				self.field:setMinParticlesPerSecond( 0 )
				self.field:setMaxParticlesPerSecond( 0 )
			end
			if self.beam ~= nil then
				-- fade out
				self.beam:setColour( xcore.xcolour( self.fade, self.fade, self.fade, self.fade ) )
				self.fade = self.fade -1
				if self.fade < 0 then self.fade = 0 end
			end
		end
	end

	if self.endpoint ~= nil then
		-- print "move effect"
		self:move()
		-- do this based on tiles?
		if self.position.X >= self.endpoint.X-1 and self.position.X <= self.endpoint.X+1 and
			self.position.Z >= self.endpoint.Z-1 and self.position.Z <= self.endpoint.Z+1 then
			-- print("[visual] endpoint reached")
			delay = 100
			return 0
		end
	end

	return 1
end

function visual:destroy ()
	-- print "[visual] destroy"
	if self.field ~= nil then
		self.field:destroy()
	end

	if self.text_field ~= nil then
		self.text_field:destroy()
	end

	if self.beam ~= nil then
		self.beam:destroy()
	end
end

function visual:setID ( n ) self.id = n end
function visual:getID () return self.id end

function visual:setParent ( n )
	self.parent_id = n
end

function visual:setText ( t )
	-- print("[visual] setText")
	self.text_field = sim:addTextNode( self.id, font_1, self.text, self.colour )
	self.text_field:setText(t)
	self.text = t
	self.text_field:setPosition( xcore.x3dpoint( self.position.X, self.position.Y, self.position.Z) )
end

function visual:setColour ( a, r, g, b )
	self.colour = xcore.xcolour( a, r, g, b )
end

function visual:setType ( n )
	-- print "[visual] setType"

	if n == VISUAL_TYPE_NONE then

	elseif n == VISUAL_TYPE_PHYSICAL then
		-- print "[visual] physical damage"
		-- create a particle system
		self.field = sim:addParticleSystem(self.id, path .. "resources/effects/fire.bmp")
		self.field:setMaterialFlag( xcore.XMF_LIGHTING, false )
		self.field:setMaterialFlag( xcore.XMF_ZWRITE_ENABLE, false )
		self.field:setMaterialType( xcore.XMT_TRANSPARENT_ADD_COLOR )
		-- self.field:setMaterialType( xcore.XMT_TRANSPARENT_VERTEX_ALPHA )

		self.field:createPointEmitter( xcore.x3dpoint(0, 0.001, 0), 10, 50,
			xcore.xcolour(255, 255, 255, 255), xcore.xcolour(255, 255, 255, 255),
			100, 1500, 360, 0.1, 2.5 )

		self.field:addFadeOutAffector( xcore.xcolour(0, 0, 0, 0), 1200 )
		-- self.field:addGravityAffector( xcore.x3dpoint(0,-0.03, 0 ), 3000 );
		self.life = 180
		
	elseif n == VISUAL_TYPE_ICE then
		-- print "[visual] VISUAL_TYPE_ICE"
		-- add a trail
		self.field = sim:addParticleSystem( self.id,  path .. "resources/effects/water.bmp")
		self.field:setMaterialFlag( xcore.XMF_LIGHTING, false )
		self.field:setMaterialFlag( xcore.XMF_ZWRITE_ENABLE, false )
		self.field:setMaterialType( xcore.XMT_TRANSPARENT_ADD_COLOR )

		self.field:createPointEmitter( xcore.x3dpoint(0, 0, 0), 10, 50,
			xcore.xcolour(255, 255, 255, 255), xcore.xcolour(100, 255, 255, 255),
			1200, 2000, 360, 3, 5 )
		self.field:addFadeOutAffector( xcore.xcolour(0, 0, 0, 0), 1000 )
		self.life = -1

	elseif n == VISUAL_TYPE_FIRE then
		-- print "[visual] VISUAL_TYPE_FIRE"
		-- create a particle system
		self.field = sim:addParticleSystem(self.id, path .. "resources/effects/fire.bmp")
		self.field:setMaterialFlag( xcore.XMF_LIGHTING, false )
		self.field:setMaterialFlag( xcore.XMF_ZWRITE_ENABLE, false )
		self.field:setMaterialType( xcore.XMT_TRANSPARENT_ADD_COLOR )
		-- self.field:setMaterialType( xcore.XMT_TRANSPARENT_VERTEX_ALPHA )

		self.field:createPointEmitter( xcore.x3dpoint(0, 0.025, 0), 100, 250,
			xcore.xcolour(255, 255, 255, 255), xcore.xcolour(255, 255, 255, 255),
			500, 1000, 360, 10, 20 )
		self.field:setPosition( xcore.x3dpoint( self.position.X, self.position.Y, self.position.Z ) )
		self.field:addFadeOutAffector( xcore.xcolour(0, 0, 0, 0), 200 )
		-- self.field:addGravityAffector( xcore.x3dpoint(0,-0.03, 0 ), 3000 );
		self.life = 300

	elseif n == VISUAL_TYPE_LIGHT then
		-- print "[visual] VISUAL_TYPE_LIGHT"
		-- create a particle system
		self.field = sim:addParticleSystem(self.id, path .. "resources/effects/smoke.bmp")
		self.field:setMaterialFlag( xcore.XMF_LIGHTING, false )
		self.field:setMaterialFlag( xcore.XMF_ZWRITE_ENABLE, false )
		self.field:setMaterialType( xcore.XMT_TRANSPARENT_ADD_COLOR )
		-- self.field:setMaterialType( xcore.XMT_TRANSPARENT_VERTEX_ALPHA )

		particle_minpps = 1000
		particle_maxpps = 1200
		particle_minlife = 1000
		particle_maxlife = 1000
		particle_angle = 0
		particle_minsize = 1
		particle_maxsize = 2
		particle_radius = 5
		particle_thickness = 1
		particle_grav = -10000
		particle_fade = 10
		particle_dir_x = 0
		particle_dir_y = 0.01
		particle_dir_z = 0
		particle_rot_x = 0
		particle_rot_y = 200
		particle_rot_z = 0
		particle_emittertype = 1
		particle_particletype = 1
		particle_is_grav = 0
		particle_is_fade = 1
		particle_is_rotate = 1
		
		self.field:addRingEmitter( xcore.x3dpoint(0, 0, 0), particle_radius, particle_thickness, xcore.x3dpoint(particle_dir_x, particle_dir_y, particle_dir_z),
			particle_minpps, particle_maxpps, xcore.xcolour(255, 255, 255, 255), xcore.xcolour(255, 255, 255, 255),
			particle_minlife, particle_maxlife, particle_angle, particle_minsize, particle_maxsize)
		self.field:setPosition( xcore.x3dpoint( self.position.X, self.position.Y, self.position.Z ) )

		-- self.field:setRotation( xcore.x3dpoint( 0, 800, 0) )
		self.field:addRotationAffector( xcore.x3dpoint(particle_rot_x, particle_rot_y, particle_rot_z), xcore.x3dpoint( self.position.X, self.position.Y, self.position.Z) )		
		self.field:addFadeOutAffector( xcore.xcolour(0, 0, 0, 0), particle_fade )

		self.life = 200
	elseif n == VISUAL_TYPE_DARK then
		-- print "[visual] VISUAL_TYPE_DARK"
		self.field = sim:addParticleSystem(self.id, path .. "resources/effects/fire.bmp")
		self.field:setMaterialFlag( xcore.XMF_LIGHTING, false )
		self.field:setMaterialFlag( xcore.XMF_ZWRITE_ENABLE, false )
		self.field:setMaterialType( xcore.XMT_TRANSPARENT_ADD_COLOR )
		-- self.field:setMaterialType( xcore.XMT_TRANSPARENT_VERTEX_ALPHA )

		particle_minpps = 1000
		particle_maxpps = 1200
		particle_minlife = 1000
		particle_maxlife = 1000
		particle_angle = 0
		particle_minsize = 5
		particle_maxsize = 10
		particle_radius = 1
		particle_thickness = 1
		particle_grav = -10000
		particle_fade = 10
		particle_dir_x = 0
		particle_dir_y = 0.1
		particle_dir_z = 0
		particle_rot_x = 0
		particle_rot_y = 200
		particle_rot_z = 0
		particle_emittertype = 1
		particle_particletype = 1
		particle_is_grav = 0
		particle_is_fade = 1
		particle_is_rotate = 1
		
		self.field:addRingEmitter( xcore.x3dpoint(0, 0, 0), particle_radius, particle_thickness, xcore.x3dpoint(particle_dir_x, particle_dir_y, particle_dir_z),
			particle_minpps, particle_maxpps, xcore.xcolour(255, 255, 255, 255), xcore.xcolour(255, 255, 255, 255),
			particle_minlife, particle_maxlife, particle_angle, particle_minsize, particle_maxsize)
		self.field:setPosition( xcore.x3dpoint( self.position.X, self.position.Y, self.position.Z ) )

		-- self.field:setRotation( xcore.x3dpoint( 0, 800, 0) )
		self.field:addRotationAffector( xcore.x3dpoint(particle_rot_x, particle_rot_y, particle_rot_z), xcore.x3dpoint( self.position.X, self.position.Y, self.position.Z) )		
		self.field:addFadeOutAffector( xcore.xcolour(0, 0, 0, 0), particle_fade )

		self.life = 200
	elseif n == VISTAL_TYPE_TEXT then
		-- print "[visual] VISTAL_TYPE_TEXT"
		-- add a text node
		self.life = 600
	end
	-- print "[visual] /setType"
end

function visual:setBeamPosition ( s, e )
	-- print( "path:" .. path )
	-- print("[visual] setBeamPosition")
	if self.beam ~= nil then
		if s ~= nil and e ~= nil then 
			self.beam:setLine( s, e, 0.2 )
		end
	end
end

function visual:getPosition ()
	return self.position
end

function visual:setPosition ( p )
	-- print "[visual] setPosition"
	self.position = p
	if self.field ~= nil then
		self.field:setPosition( xcore.x3dpoint( p.X, p.Y, p.Z ) )
	end

	if self.text_field ~= nil then
		self.text_field:setPosition( xcore.x3dpoint( p.X, p.Y, p.Z) )
	end
	-- print "[visual] /setPosition"
end

function visual:setEndPoint ( p )
	-- print "[visual] setEndPoint"
	self.endpoint = p

	xdif = self.position.X - self.endpoint.X
	ydif = self.position.Z - self.endpoint.Z

	x = xdif
	y = ydif*-1
	
	self.angle = math.atan(y/x)/(math.pi/180)
	if x<0 then
		self.angle = self.angle + 180
	end
	if x>=0 and y<0 then
		self.angle = self.angle + 360
	end

	self.speed = 0.25
	self.r = self.angle * math.pi/180;
	-- print "[visual] /setEndPoint"
end

function visual:move()
	-- print "[visual] move"
	local xspeed = math.cos( self.r ) * self.speed
	local yspeed = math.sin( self.r ) * self.speed
	-- print ( "[visual] r:" .. self.r )
	-- print ( "[visual] xspeed:" .. xspeed )
	-- print ( "[visual] xspeed:" .. yspeed )
	-- print ( "[visual] self.position:" .. self.position.X .. ", " .. self.position.Z )
	self:setPosition( xcore.x3dpoint( self.position.X - xspeed, self.position.Y, self.position.Z + yspeed ) )
end
