-- xenjine layer 2 network tutorial 2b (p-shooter server)
-- c2008-2009 lukas irwin | sysop [ at ] xhrit.com
-- tutorial_shooter_server.module

print "[server] p-shooter server demo"

path = "modules/shooter_server/"
function doload( f )
	-- print ("doing load ov file " .. path.. f )
	x = loadfile ( path .. f )
	x()
end

doload( "connection.lua" )
doload( "user.lua" )
doload( "pawn.lua" )

-- server
function init()
	print "[server] init server"
	-- create net object and init it as a server
	net = xcore.xnet()
	net:server()

	-- create the sql object and make the user table
	-- if the user table already exists the create table call will fail.
	-- this is ok, as that means the table will not be over-written everytime you restart
	sql = xcore.xsql( path .. "resources/auth.db" )
	sql:exe( "CREATE TABLE users (id INTEGER PRIMARY KEY, name text, password text, score int)" )

	-- start the physics
	dyn = xcore.xdynamics()
	dyn:setAutoDisable( false )
	dyn:setERP ( 0.8 )
	dyn:setCFM ( 0.001 )
	dyn:setTimeStep( 0.01 )
	dyn:setMode ( xcore.XDYNAMICS_FAST )

	-- create a planet
	np = xcore.x3dpoint( 300, -20, 0 )
	planet_body = xcore.xbody() -- sim:addBody ( 0, mesh_file, tex_file )??
	dyn:createBody( planet_body )
	dyn:createSphereGeom( planet_body, 200, np, xcore.x3dpoint( 0, 0, 0 ) ) -- size, pos, rot

	time = xcore.xtimer()

	-- data tables
	connections = {}
	users = {}
	objects = {}
	connection_sequence = 0
	object_sequence = 0
	-- print "[server] /init"
end

function update()
	-- print "[server] update network"
	net:update()
	dyn:update()
	time:update()

	--use a while statement to pull as many packets as you can
	packet = net:get()
	if packet ~= nil then
		-- print "[server] packet not null"
		-- print ( "[server] packet type : " .. tonumber( packet:getType() ) )
		if tonumber( packet:getType() ) == xcore.XPACKET_CONNECT then
			-- hmm... seems this is not being transmitted when client is connecting to server
			print "[server] packet connect"
			-- send a connect notice
			if getConnectionIndexByPacketID( packet:getID() ) ~= nil then 
				i = getUserIndexByConnectionID( connections[ getConnectionIndexByPacketID( packet:getID() )]:getID() )
				if i ~= nil then
					-- id = users[i]:getID()
					m = users[i]:getName() .. " has connected."
					net:send( xcore.XPACKET_MESSAGE, m )
				end
			end
			
		elseif tonumber( packet:getType() ) == xcore.XPACKET_VALIDATE then
			print "[server] packet validate"
			
			-- add client id to connection array, with status ov non-valid? (this should be in packet connect)
			connection_sequence = connection_sequence + 1
			print "[server] spawn connection"
			i = table.getn( connections ) + 1
			connections[ i ] = connection:new()
			connections[ i ]:setID( connection_sequence )
			connections[ i ]:setPacketID( packet:getID() )
			connections[ i ]:setState( 0 )
			print "[server] connection spawned"
			
			-- validate? strip into un and pw
			i = string.find ( packet:getMessage(), ":")
			n = string.sub ( packet:getMessage(), 1, i-1 )
			p = string.sub ( packet:getMessage(), i+1, string.len( packet:getMessage() ) )

			-- check to see if user is already logged in
			if getUserIndexByName( n ) ~= nil then
				print "user already logged in."
				net:sendTo( xcore.XPACKET_AUTH, "denied_in_use", packet:getID() )
			else
				-- check to see if name exists in database
				q = sql:exe( "SELECT * FROM users WHERE name = '" .. n .. "'" )
				-- sql:printAll()
				if q ~= nil then
					if q:getRowCount() > 0 then
						print "[server] found existing user"
						-- name found, check to see if passwords match
						sql:removeResult( q )
						q = sql:exe( "SELECT * FROM users WHERE name = '" .. n .. "' AND password = '" .. p .. "'" )
						if q ~= nil then 
							if q:getRowCount() > 0 then -- TODO interesting side effect of returnint q object is if there are 0 rows you get a null object. hmm, need to fix this.
								print "[server] passwords match, spawning user."
								-- if yes, then auth
								-- spawn user
								print ( "head : " .. q:getHead( 0 ) )
								print ( "data : " .. q:getData( 0 ) )
								user_id = q:getData( 0 )
								user_score = q:getData( 3 )
								net:sendTo( xcore.XPACKET_AUTH, "allow;" .. user_id, packet:getID() )
								connections[ getConnectionIndexByPacketID( packet:getID() ) ]:setState( 2 )
								connection_id = connections[ getConnectionIndexByPacketID( packet:getID() ) ]:getID()
								spawnUser( user_id, n, connection_id, user_score )
								print "[server] user spawned"
							else
								print "[server] passwords do not match"
								-- if no, then deny
								net:sendTo( xcore.XPACKET_AUTH, "denied_invalid", packet:getID() )
								-- TODO : reject connection?
							end
							sql:removeResult( q )
						else
							print "[server] passwords do not match"
							-- if no, then deny
							net:sendTo( xcore.XPACKET_AUTH, "denied_invalid", packet:getID() )
							-- TODO : reject connection?
						end
					else
						print "[server] create new user"
						-- name not found, add name and pw to database and send auth
						sql:exe( "INSERT INTO users (name, password, score) VALUES ('" .. n .. "', '" .. p .. "', 0)" )
						-- get id
						q = sql:exe( "SELECT * FROM users WHERE name = '" .. n .. "'" )
						-- spawn user
						-- print ( "head : " .. q:getHead( 0 ) )
						-- print ( "data : " .. q:getData( 0 ) )
						user_id = q:getData( 0 )
						net:sendTo( xcore.XPACKET_AUTH, "allow;" .. user_id, packet:getID() )
						connections[ getConnectionIndexByPacketID( packet:getID() ) ]:setState( 2 )
						connection_id = connections[ getConnectionIndexByPacketID( packet:getID() ) ]:getID()
						spawnUser( user_id, n, connection_id, 0 )
						sql:removeResult( q )
						print "[server] user spawned"
					end
				else
					-- fix this redundent code...
					print "[server] create new user"
					-- name not found, add name and pw to database and send auth
					sql:exe( "INSERT INTO users (name, password, score) VALUES ('" .. n .. "', '" .. p .. "', 0)" )
					-- get id
					q = sql:exe( "SELECT * FROM users WHERE name = '" .. n .. "'" )
					-- spawn user
					-- print ( "head : " .. q:getHead( 0 ) )
					-- print ( "data : " .. q:getData( 0 ) )
					user_id = q:getData( 0 )
					net:sendTo( xcore.XPACKET_AUTH, "allow;" .. user_id, packet:getID() )
					connections[ getConnectionIndexByPacketID( packet:getID() ) ]:setState( 2 )
					connection_id = connections[ getConnectionIndexByPacketID( packet:getID() ) ]:getID()
					spawnUser( user_id, n, connection_id, 0 )
					sql:removeResult( q )
					print "[server] user spawned"
				end
			end

			-- debug output
			listUsers()
			listObjects()
			listConnections()
			
		elseif tonumber( packet:getType() ) == xcore.XPACKET_DISCONNECT then
			print "[server] packet disconnect"
			-- remove user from server's user table
			if getConnectionIndexByPacketID( packet:getID() ) ~= nil then 
				i = getUserIndexByConnectionID( connections[ getConnectionIndexByPacketID( packet:getID() )]:getID() )
				if i ~= nil then
					-- update user's score
					query = "UPDATE users SET score = " .. users[i]:getScore() .. " WHERE id = ".. users[i]:getID()
					print( query )
					sql:exe( query )

					-- send a disconnect notice
					id = users[i]:getID()
					m = users[i]:getName() .. " has disconnected"
					net:send( xcore.XPACKET_MESSAGE, m )
					
					-- remove user from user table
					table.remove ( users, i )

					-- tell clients to remove user
					net:send( xcore.XPACKET_USER_PART, id )
					
					-- delete the user's objects
					for i=object_count,1,-1 do
						if objects[i]:getOwner() == id then
							-- TODO: mark for delete instead of delete?
							despawnObject( i )
						end
					end
					
					-- remove from connections table
					print "[server] drop connection"
					i = getConnectionIndexByPacketID( packet:getID() )
					table.remove ( connections, i )
					-- print "[server] /packet disconnect"

					-- show debug info
					-- listUsers()
					-- listObjects()
					-- listConnections()
				end
			else
				print "user didn't validate"
			end
		elseif tonumber( packet:getType() ) == xcore.XPACKET_MESSAGE then
			print "[server] packet message"
			a = packet:getMessage()
			b = packet:getID()
			i = getUserIndexByConnectionID( connections[ getConnectionIndexByPacketID( packet:getID() ) ]:getID() )
			c = users[i]:getName() .. " : " .. a
			
			-- TODO: FIXME!!! lol. use a wrapper function to check validation, wrap send in sendToValidated?			
			-- loop over connected array, sending only to user ids that have been validated.
			-- i = table.getn( connections )
			-- print ( "[server] num connections " .. i )
			-- for n = 1, i do
				-- print ( "[server] connections[n] " .. connections[n] )
			-- 	if connections[n]:getState() == 2 then
			-- 		net:sendTo( xcore.XPACKET_MESSAGE, c, connections[n]:getPacketID() )
			-- 	end
			-- end
			
			net:send( xcore.XPACKET_MESSAGE, c )
	
		elseif tonumber( packet:getType() ) == xcore.XPACKET_ACTION then
			-- print "\n[server] use item"
			msg = packet:getMessage()
			-- print ("[server] msg:" .. msg )

			user_index = getUserIndexByConnectionID( connections[ getConnectionIndexByPacketID( packet:getID() ) ]:getID() )
			uid = users[ user_index ]:getID()
			-- print ("[server] user_id:" .. uid )
			-- has shot a lazor, since that is all we can do for now.

			focused_object_index = getObjectIndexByID( users[ user_index ]:getFocusedObjectID() )
			if focused_object_index ~= nil then 
				if objects[ focused_object_index ]:getDelay() == 0 then
					objects[ focused_object_index ]:setDelay( 50 )
					-- get user owned object
					user_pos = objects[ focused_object_index ]:getPosition()
					user_rot = objects[ focused_object_index ]:getRotation()

					m = xcore.xmatrix4()
					endpoint = xcore.x3dpoint( 0, 10, 0 )
					m:setRotationDegrees( user_rot )
					m:transformVect( endpoint )
					
					startpoint = user_pos
					endpoint.X = endpoint.X + startpoint.X
					endpoint.Y = endpoint.Y + startpoint.Y
					endpoint.Z = endpoint.Z + startpoint.Z

					-- create :type, owner
					-- print "[server] spawning effect"
		
					id = spawnObject( uid, 2, endpoint, user_rot )
					m = "" .. id .. ";2;" .. uid ..";"
					-- print ( "[server] sending create data : " .. m )

					-- add object's initial force
					i = getObjectIndexByID( id )
					objects[ i ]:getBody():addRelForce( xcore.x3dpoint( 0, 10000, 0 ) );

					net:send( xcore.XPACKET_OBJECT_ADD, m )
				else
					-- print "action too soon"
				end
			end
			
		elseif tonumber( packet:getType() ) == xcore.XPACKET_OBJECT_UPDATE_POSITION then
			-- print "[server] packet position"
			dir = packet:getMessage()
			-- print ("dir:" .. dir )
			-- print ("packet id:" .. packet:getID() )
			-- print ("connection id:" .. connections[ getConnectionIndexByPacketID( packet:getID() ) ]:getID() )

			user_index = getUserIndexByConnectionID( connections[ getConnectionIndexByPacketID( packet:getID() ) ]:getID() )
			uid = users[ user_index ]:getID()

			focused_object_index = getObjectIndexByID( users[ user_index ]:getFocusedObjectID() )
			if focused_object_index ~= nil then 
				p = objects[ focused_object_index ]:getPosition()
				
				if dir == "1" then
					-- objects[i]:setPosition( xcore.x3dpoint(p.X - 1, p.Y, p.Z ) )
					objects[ focused_object_index ]:getBody():addRelForceAtRelPos( xcore.x3dpoint( 400000,0,0 ), xcore.x3dpoint( 0, -10, 0 ) );
					-- print "[server] /turn left"
				elseif dir == "2" then
					-- objects[i]:setPosition( xcore.x3dpoint(p.X + 1, p.Y, p.Z ) )
					objects[ focused_object_index ]:getBody():addRelForceAtRelPos( xcore.x3dpoint( -400000,0,0 ), xcore.x3dpoint( 0, -10, 0 ) );
					-- print "[server] /turn right"
				elseif dir == "3" then
					-- objects[i]:setPosition( xcore.x3dpoint(p.X, p.Y, p.Z +1 ) )
					objects[ focused_object_index ]:getBody():addRelForce( xcore.x3dpoint( 0,3200000,0 ) );
					-- print "[server] /move forward"
				elseif dir == "4" then
					-- objects[i]:setPosition ( xcore.x3dpoint(p.X, p.Y, p.Z -1 ) )
					objects[ focused_object_index ]:getBody():addRelForce( xcore.x3dpoint( 0,-3200000,0 ) );
					-- print "[server] /move back"
				else
					-- unknown direction?
				end
				-- print "ok 2."
				-- p = objects[i]:getPosition()
				-- p:trace()
				-- print "[server] /packet position"
			end
		else
			print ( "[server] unknown packet type #: " .. tonumber( packet:getType() ) )
		end
	end

	updateGame()
end

-- required, but not used as server operates headless.
function render() end

function updateGame()
	-- print "[server] updateGame"
	-- loop the object array and test for collisions
	-- do any create events, for new users and respawns
	user_count = table.getn( users )
	for lp_a = 1, user_count do
		if users[ lp_a ]:getInit() == 0 then
			print "\n[server] init user"
			users[ lp_a ]:setInit( 1 )
			uid = users[ lp_a ]:getID()
			packet_id = connections[ getConnectionIndexByConnectionID( users[ lp_a ]:getConnectionID() ) ]:getPacketID()

			uname = users[ lp_a ]:getName()
			uscore = users[ lp_a ]:getScore()
			-- sendto new user a create message for all objects

			-- add a create message to outgoing message que
			object_count = table.getn( objects )
			print ( "[server] synching " .. object_count .. " objects" )
			for lp_b = 1, object_count do
				pid = objects[ lp_b ]:getID()
				pt = objects[ lp_b ]:getType()
				po = objects[ lp_b ]:getOwner()
				m = "" .. pid .. ";" ..  pt .. ";" .. po
				net:sendTo( xcore.XPACKET_OBJECT_ADD, m, packet_id )

				ppos = objects[ lp_b ]:getPosition()
				m = "" .. pid .. ";" .. ppos.X .. ";" .. ppos.Y .. ";" .. ppos.Z
				net:sendTo( xcore.XPACKET_OBJECT_UPDATE_POSITION, m, packet_id ) -- getConnectionID
				-- TODO: should update rotation too
				print ( "[server] synched object " .. lp_b )
			end

			-- create the new user's ship at a random position and rotation.
			randx = math.random(-100, 100)
			randy = math.random(-100, 100)
			id = spawnObject( uid, 1, xcore.x3dpoint( randx, randy, 0 ), xcore.x3dpoint( 0, 0, 0 ) )
			m = "" .. id .. ";1;" .. uid ..";"
			-- print ( "[server] sending create data : " .. m )
			net:send( xcore.XPACKET_OBJECT_ADD, m )
			print ("[server] set new user's focused object id to: " .. id)
			users[ lp_a ]:setFocusedObjectID( id )
			print "[server] /init user"

			-- debug output
			listUsers()
			listObjects()
			listConnections()
		end
		if users[ lp_a ]:getInit() == 2 then
			print "\n[server] respawn user"		
			-- respawn
			users[ lp_a ]:setInit( 1 )
			uid = users[ lp_a ]:getID()
			packet_id = connections[ getConnectionIndexByConnectionID( users[ lp_a ]:getConnectionID() ) ]:getPacketID()

			uname = users[ lp_a ]:getName()
			uscore = users[ lp_a ]:getScore()

			-- create the user's new ship.
			randx = math.random(-100, 100)
			randy = math.random(-100, 100)			
			id = spawnObject( uid, 1, xcore.x3dpoint( randx, randy, 0 ), xcore.x3dpoint( 0, 0, 0 ) )
			m = "" .. id .. ";1;" .. uid ..";"
			-- print ( "[server] sending create data : " .. m )
			net:send( xcore.XPACKET_OBJECT_ADD, m )
			print ("[server] set new user's focused object id to: " .. id)
			users[ lp_a ]:setFocusedObjectID( id )
			print "[server] /init user"			

			-- debug output
			listUsers()
			listObjects()
			listConnections()
		end
	end

	-- check for collisions and remove any destroyed objects
	checkCollision()
	
	-- print "check for expired objects"
	object_count = table.getn( objects )
	for i=object_count,1,-1 do
		if objects[i]:getRemove() == 1 then
			despawnObject ( i )
		end
	end
	
	-- send game updates
	object_count = table.getn( objects )
	-- print ( "[server] num objects " .. i )
	for n = 1, object_count do
		b = objects[n]:getBody()
		b:alignToAxis()
		if objects[n]:getType() == 1 then b:damp(0.01) end
		-- update objects (?)
		objects[n]:update()

		-- polling interval;
		if objects[n]:shouldPoll() == true then
			-- print "[server] update object"
			-- objects[n]:setLastPoll( time:getTime() ) -- now

			p = objects[n]:getPosition()
			objects[n]:setLastPos( p )

			m = "" .. objects[n]:getID() .. ";" .. p.X .. ";" .. p.Y .. ";" .. p.Z
			-- print ( "[server] sending positional data : " .. m )
			net:send( xcore.XPACKET_OBJECT_UPDATE_POSITION, m )

			r = objects[n]:getRotation()
			objects[n]:setLastRot( r )
			
			m = "" .. objects[n]:getID() .. ";" .. r.X .. ";" .. r.Y .. ";" .. r.Z
			-- print ( "[server] sending rotation data : " .. m )
			net:send( xcore.XPACKET_OBJECT_UPDATE_ROTATION, m )
		end
	end
	-- print "[server] updateGame /end"
end

function checkCollision()
	-- print ( "[server] checkCollision")
	local ct = dyn:getCollisionCount()
	-- print ( "[server] collision count" .. ct )

	for n = 1, ct do
		local c = dyn:getCollision()  
		-- while c ~= nil do
		if c ~= nil then
			-- print "[server] valid collision, test for objects"
			local c1 = c:geto1()
			local c2 = c:geto2()
			local c1_index = getObjectIndexByBodyID( c1:getID() )
			local c2_index = getObjectIndexByBodyID( c2:getID() )
			if c1_index ~= nil and c2_index ~= nil then
				if tonumber(objects[ c1_index ]:getType()) ~= 1 or tonumber(objects[ c2_index ]:getType()) ~= 1 then
					print ( "[server] collision between objects at index " .. c1_index .. " and " .. c2_index )
					-- print ( "[server] o1 : " .. objects[ c1_index ]:getName() .. ", dam " .. objects[ c1_index ].damage_armor )
					-- print ( "[server] o2 : " .. objects[ c2_index ]:getName() .. ", dam " .. objects[ c2_index ].damage_armor )
					objects[ c1_index ].damaged_by = objects[ c2_index ]:getOwner()
					objects[ c2_index ].damaged_by = objects[ c1_index ]:getOwner()
					
					objects[ c1_index ]:setRemove( 1 )
					objects[ c2_index ]:setRemove( 1 )
					
				end
			else
				-- print "[server] error; collision w/ null index"
				-- if c1_index == nil then print ("[server] body_id# " .. c1:getID() .. "/c1_index is null") end
				-- if c2_index == nil then print ("[server] body_id# " .. c2:getID().. "/c2_index is null") end
				
				-- self:debugInfo()
			end
		else
			-- print "[server] no collisions"
		end
		dyn:removeCollision()
	end
	return 1
end 

function spawnObject( o, t, p, r) -- ownerid, type, pos, rot
	-- print "[server] spawnObject"
	object_sequence = object_sequence + 1
	-- print ( "[server] object_sequence = " .. object_sequence )
	a = pawn:new()
	i = table.getn( objects ) + 1
	objects[ i ] = a

	objects[ i ]:setID ( object_sequence )
	objects[ i ]:setOwner ( o )
	objects[ i ]:setType ( t )
	objects[ i ]:setPosition ( p )
	objects[ i ]:setRotation ( r )
	objects[ i ]:setLastPoll ( os.clock() )

	-- print ("[server] object id :" .. object_count)
	-- print ("[server] owner id :" .. o)
	-- print ("object rot--------------")
	-- r:trace()

	-- create geometry for player ship
	body = xcore.xbody() -- sim:addBody ( 0, mesh_file, tex_file )??
	dyn:createBody( body )
	if t == 1 then
		dyn:createBoxGeom( body, xcore.x3dpoint(10, 10, 10), p, r ) -- size, pos, rot
	else
		dyn:createBoxGeom( body, xcore.x3dpoint(1, 1, 1), p, r ) -- size, pos, rot
	end

	if t == 2 then
		objects[ i ]:setLifeSpan( 500 )
	end
	
	body:setMU( 100 )
	dyn:setPlane2d( body );
	objects[ i ]:setBody( body )

	-- print "[server] end spawn object"

	return object_sequence;
end

function despawnObject( index ) -- object index
	-- print "\n[server] despawnObject"

	-- adjust score
	if tonumber( objects[ index ]:getType() ) == 1 then -- ENUM_OBJECT_TYPE_ORGANIC
		-- print ( "[server] dead player, setting init 2" )
		oid = objects[ index ]:getOwner()
		if oid ~= nil then
			oindex = getUserIndexByID( oid )
			if oindex ~= nil then users[ oindex ]:setInit( 2 ) end
		end
		if objects[ index ].damaged_by ~= nil then
			-- print ( "[server] user " .. users[ oindex ]:getName() .. " killed by " .. users[ getUserIndexByID( objects[ index ].damaged_by  ) ]:getName() )
			users[ getUserIndexByID( objects[ index ].damaged_by  ) ].score = users[ getUserIndexByID( objects[ index ].damaged_by  ) ].score + 1
			-- TODO: send score adjustment packet?
			-- print "send new score out"
			data = "score; " .. users[ getUserIndexByID( objects[ index ].damaged_by  ) ].score
			user_id = objects[ index ].damaged_by
			user_index = getUserIndexByID( objects[ index ].damaged_by  )
			packet_id = connections[ getConnectionIndexByConnectionID( users[ user_index ]:getConnectionID() ) ]:getPacketID()
			
			net:sendTo( xcore.XPACKET_USER_UPDATE, data, packet_id )
			-- print "/end ??"
		end
	end
	
	-- remove object
	oid = objects[ index ]:getID()
	dyn:destroyJoints( objects[ index ]:getBody() )
	dyn:destroyGeom( objects[ index ]:getBody() )
	dyn:destroyBody( objects[ index ]:getBody() )
	table.remove ( objects, index )

	-- tell connected clients to delete stuff too.
	-- print "[server] tell clients to delete objects"
	net:send( xcore.XPACKET_OBJECT_DELETE, oid )

	-- check to see if object was focused
	-- TODO: not sure if this is the proper place for this code
	i = table.getn( users )
	-- print ( "[server] users.count = " .. i )
	for n = 1, i do
		if users[ i ]:getFocusedObjectID() == oid then
			users[ i ]:setFocusedObjectID( -1 )
			-- let user know their object has been destroyed?
			-- stick them in a respawn que?
		end
	end
	
	-- print "[state_game] /end despawnObject"
end

function spawnUser( uid, n, cid, s ) -- user_id, name, connection_id, score
	print "[server] spawn user"
	-- add user to user array
	i = table.getn( users ) + 1
	print ( "[server] name :" .. n )
	print ( "[server] user id:" .. uid )
	print ( "[server] connection id:" .. cid )
	users[ i ] = user:new()
	users[ i ]:setConnectionID( cid )
	users[ i ]:setID( uid )
	users[ i ]:setName( n )
	users[ i ]:setScore( s )	
	users[ i ]:setInit( 0 )

	data = "score; " .. s
	packet_id = connections[ getConnectionIndexByConnectionID( cid ) ]:getPacketID()
	net:sendTo( xcore.XPACKET_USER_UPDATE, data, packet_id )
	
	print "[server] end spawn user"
end

function getObjectIndexByOwnerID( oid ) -- owner, type, pos, rot
	-- print "[server] getObjectIndexByOwnerID"
	i = table.getn( objects )
	-- print ( "[server] objects.count = " .. i )
	for n = 1, i do
		-- print ("object " .. n .. " owner:" .. objects[ n ]:getOwner())
		if  objects[ n ]:getOwner() == oid then
			-- print ( "[server] return " .. n )
			return n
		end
	end
	-- print "[server] return null"
	return nil
end

function getConnectionIndexByPacketID( pid )
	-- print "[server] getConnectionIndexByPacketID"
	i = table.getn( connections )
	-- print ( "[server] connections.count = " .. i )
	for n = 1, i do
		-- print ("[server] connection id:" .. connections[ n ]:getConnectionID() )
		if  connections[ n ]:getPacketID() == pid then
			-- print ( "[server] return " .. n )
			return n
		end
	end
	-- print "[server] return null"
	return nil
end

function getConnectionIndexByConnectionID( cid )
	-- print "[server] getConnectionIndexByPacketID"
	i = table.getn( connections )
	-- print ( "[server] connections.count = " .. i )
	for n = 1, i do
		-- print ("[server] connection id:" .. connections[ n ]:getConnectionID() )
		if  connections[ n ]:getID() == cid then
			-- print ( "[server] return " .. n )
			return n
		end
	end
	-- print "[server] return null"
	return nil
end

function getUserIndexByConnectionID( cid )
	-- print "[server] getUserIndexByConnectionID"
	i = table.getn( users )
	-- print ( "[server] users.count = " .. i )
	for n = 1, i do
		-- print ("[server] user # " .. n .. ", " .. users[ n ]:getName() )
		-- print ("[server] connection id:" .. users[ n ]:getConnectionID() )
		if  users[ n ]:getConnectionID() == cid then
			-- print ( "[server] return " .. n )
			return n
		end
	end
	-- print "[server] return null"
	return nil
end

function getUserIndexByName( un ) -- owner, type, pos, rot
	-- print ( "[server] getUserIndexByName(" .. un .. ")" )
	i = table.getn( users )
	for n = 1, i do
		-- print ("user #" .. n .. " name:" .. users[ n ]:getName() )
		if  users[ n ]:getName() == un then
			-- print ( "[server] return " .. n )
			return n
		end
	end
	-- print "[server] return null"
	return nil
end

function getUserIndexByID( id ) -- owner, type, pos, rot
	-- print ( "[server] getUserIndexByID(" .. id  .. ")" )
	i = table.getn( users )
	for n = 1, i do
		-- print ("user #" .. n .. " id:" .. users[ n ]:getID() )
		if  users[ n ]:getID() == id then
			-- print ( "[server] return " .. n )
			return n
		end
	end
	-- print "[server] return null"
	return nil
end

function getObjectIndexByID( id )
	-- print "[server] getObjectIndexByID"
	i = table.getn( objects )
	-- print ( "[server] objects.count = " .. i )
	for n = 1, i do
		-- print ("[server] object " .. n .. " id:" .. objects[ n ]:getID())
		-- print ("[server] match against id : " .. id)
		if  tonumber( objects[ n ]:getID() ) == tonumber( id ) then
			-- print ( "[server] return " .. n )
			return n
		end
	end
	-- print "[server] return null"
	return nil
end

function getObjectIndexByBodyID( id )
	-- print "[server] getObjectIndexByOwnerID"
	local i = table.getn( objects )
	-- print ( "[server] objects.count = " .. i )
	for n = 1, i do
		-- print ("object " .. n .. " owner:" .. objects[ n ]:getOwner())
		if  objects[ n ]:getBody():getID() == id then
			-- print ( "[server] return " .. n )
			return n
		end
	end
	-- print "[server] return null"
	return nil
end

-- here are some debugging functions

function listUsers()
	print ("\nListing Users" )
	user_count = table.getn( users )
	for lp_a = 1, user_count do
		print ("------------------------------------" )
		print ("[server] id:" .. users[ lp_a ]:getID() )
		print ("[server] name:" .. users[ lp_a ]:getName() )
		print ("[server] connection id:" .. users[ lp_a ]:getConnectionID() )
		print ("[server] focused oject id:" .. users[ lp_a ]:getFocusedObjectID() )
		print ("[server] init:" .. users[ lp_a ]:getInit() )
		print ("[server] score:" .. users[ lp_a ]:getScore() )
	end
end

function listObjects()
	print ("\nListing Objects" )
	object_count = table.getn( objects )
	for lp_a = 1, object_count do
		print ("------------------------------------" )
		print ("[server] id:" .. objects[ lp_a ]:getID() )
		print ("[server] owner:" .. objects[ lp_a ]:getOwner() )
		print ("[server] type:" .. objects[ lp_a ]:getType() )
	end
end	

function listConnections()
	print ("\nListing Connections" )
	connection_count = table.getn( connections )
	for lp_a = 1, connection_count do
		print ("------------------------------------" )
		print ("[server] id:" .. connections[ lp_a ]:getID() )
		print ("[server] packet id:" .. connections[ lp_a ]:getPacketID() )
		print ("[server] state:" .. connections[ lp_a ]:getState() )
	end
end	

-- eof