-- xenjine graphics tutorial 4
-- c2007-2012 lukas irwin
-- tutorial_graphics_4.lua
-- materials, shaders & light effects

-- required
function init()
	-- start a simulation
	-- require "module.settings" for video options
	sim = xcore.xsimulation ( xcore.XDT_OPENGL , 1440, 900, 32, false, true, true )

	sim:setWindowCaption( "xenjine graphics demo 11 : Per-Pixel Lighting" )

	-- add some static text
	sim:addText( 0, "xenjine graphics demo 11 : Per-Pixel Lighting", 50, 50, 300, 62)

	-- add a static camera
	cam = sim:addCamera ( 1, xcore.CAMERA_FPS, -413, 305, -270, 0,0,0)
	sim:bindkey ( "quit",  xcore.XKEY_ESCAPE  ) -- exit demo
	sim:bindkey ( "zoom_in",  xcore.XKEY_PLUS  )
	sim:bindkey ( "zoom_out",  xcore.XKEY_MINUS  )

	-- add the sun
	sun = sim:addMesh ( 2, "modules/tutorial_graphics_4/resources/earth.x" )
	sun:addTexture( "modules/tutorial_graphics_4/resources/sunmap.jpg", 0 )
	-- earth = sim:addSphereMesh ( 2, "modules/tutorial_graphics_4/resources/earthmap1k.jpg" )
	sun:setScale( xcore.x3dpoint( 100, 100, 100 ) )
	sun:setMaterialFlag( xcore.XMF_LIGHTING, false )
	sun:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER , true )
	sun:setMaterialFlag( xcore.XMF_BACK_FACE_CULLING, false )

	-- add some shaders
	-- shader = sim:addShader()
	-- s_mat1_1 = shader:addMaterial( "shaders/bloom.frag", "shaders/bloom.vert", xcore.XMT_SOLID )

	c = xcore.xcolour( 255, 255, 255, 255 )
	-- sun:setMaterialType( s_mat1_1 )

	-- add the light ov the sun
	light = sim:addLight( 777, xcore.x3dpoint( 0, 0, 0 ), xcore.xcolour( 255, 255, 255, 255 ), 1500 )

	-- add the earth
	planet_size = 10
	earth = sim:addMesh ( 2, "modules/tutorial_graphics_4/resources/earth.x" )
	earth:addTexture( "modules/tutorial_graphics_4/resources/earthmap1k.jpg", 0 )
	earth:setScale( xcore.x3dpoint( planet_size, planet_size, planet_size ) )
	-- earth:setMaterialFlag( xcore.XMF_LIGHTING, false )
	earth:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER , true )
	earth:setPosition( xcore.x3dpoint( 1000, 0, 0 ) )

	-- add a bump map
	earth:addNormalMap( "modules/tutorial_graphics_4/resources/earthbump1k.jpg", 20 )
	c = xcore.xcolour( 255, 255, 255, 255 )
	-- earth:setMaterialType( xcore.XMT_NORMAL_MAP_TRANSPARENT_VERTEX_ALPHA, c, c, c  )
	earth:setMaterialType( xcore.XMT_PARALLAX_MAP_SOLID )
	-- earth:setMaterialType( xcore.XMT_NORMAL_MAP_SOLID, c, c, c  )
	-- earth:setMaterialType( s_mat1_3, c, c, c  )
	-- earth:setMaterialFlag( xcore.XMF_NORMALIZE_NORMALS , true )

	-- add a spec map
	-- earth:addTexture( "modules/tutorial_graphics_4/resources/earthspec1k.jpg", 2 )
	-- earth:setMaterialType( xcore.XMT_LIGHTMAP_LIGHTING , c, c, c  )
	-- earth:setMaterialType( xcore.XMT_LIGHTMAP_LIGHTING_M4 , c, c, c  )

	-- add a transparent cloud layer
	atmosphere_size = planet_size * 1.025
	print ("planet_size:" .. planet_size)
	print ("atmosphere_size:" .. atmosphere_size)
	earth_atmosphere = sim:addMesh ( 3, "modules/tutorial_graphics_4/resources/earth.x" )
	earth_atmosphere:addTexture( "modules/tutorial_graphics_4/resources/earthcloudmap.tga", 0 )
	earth_atmosphere:setScale( xcore.x3dpoint( atmosphere_size, atmosphere_size, atmosphere_size ) )
	earth_atmosphere:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER , true )
	-- earth_atmosphere:setMaterialFlag( xcore.XMF_BACK_FACE_CULLING, false )
	c = xcore.xcolour( 255, 255, 255, 255 )
	earth_atmosphere:setMaterialType( xcore.XMT_TRANSPARENT_ALPHA_CHANNEL )
	-- earth_atmosphere:setMaterialType( s_mat1_2, c, c, c  )
	earth_atmosphere:setPosition( xcore.x3dpoint( 1000, 0, 0 ) )

	-- add corona layer
	-- earth_corona = sim:addMesh ( 3, "modules/tutorial_graphics_4/resources/earth.x", "modules/tutorial_graphics_4/resources/earthcloudmap.tga" )
	-- earth_corona:setScale( xcore.x3dpoint( 12.8, 12.8, 12.8 ) )
	-- c = xcore.xcolour( 255, 255, 255, 255 )
	-- earth_corona:setMaterialType( s_mat1_3 , c, c, c  )
	-- earth_corona:setPosition( xcore.x3dpoint( 3000, 0, 0 ) )

	-- moon
	moon = sim:addMesh ( 2, "modules/tutorial_graphics_4/resources/earth.x" )
	moon:addTexture( "modules/tutorial_graphics_4/resources/moonmap1k.jpg", 0)
	moon:setScale( xcore.x3dpoint( 5, 5, 5 ) )
	moon:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER , true )
	moon:setPosition( xcore.x3dpoint( 1000, 0, 100 ) )

	-- add a bump map
	moon:addNormalMap( "modules/tutorial_graphics_4/resources/moonbump1k.jpg", 5 )
	c = xcore.xcolour( 0, 0, 0, 0 )
	moon:setMaterialType( xcore.XMT_PARALLAX_MAP_SOLID )
	moon:setMaterialFlag( xcore.XMF_NORMALIZE_NORMALS , true )

	-- add a skybox
	sky = sim:addSkybox(500, "modules/tutorial_graphics_4/resources/starfield.jpg",
			"modules/tutorial_graphics_4/resources/starfield.jpg",
			"modules/tutorial_graphics_4/resources/starfield.jpg",
			"modules/tutorial_graphics_4/resources/starfield.jpg",
			"modules/tutorial_graphics_4/resources/starfield.jpg",
			 "modules/tutorial_graphics_4/resources/starfield.jpg")
	zoom = 50

	-- set the camera aspect and fov
	cam:setFOV( 45 * 3.14159265 / 180 )
	cam:setAspectRatio( 16/9 )
	cam:setFarValue( 40000 )

	-- tag something
	font = sim:getBuiltInFont()
	-- label = sim:addTextNode( 666, font, "Sol", xcore.xcolour( 255, 255, 255, 0 ) )
	-- label2 = sim:addTextNode( 666, font, "Terra", xcore.xcolour( 255, 255, 0, 255 ) )
	-- label3 = sim:addTextNode( 666, font, "Luna", xcore.xcolour( 255, 255, 0, 255 ) )
	-- label2:setPosition( xcore.x3dpoint( 1000, 0, 0) )
	-- label3:setPosition( xcore.x3dpoint( 1000, 0, 100) )
	-- print "/end?"
	sim:setAmbientLight( xcore.xcolour( 255, 20, 20, 20 ) )
	-- sim:initBloom( 0.01, 10, 5 )
	-- sim:initBloom( 0.008, 3, 5 )

	-- sim:initBlur( 0.9 )
	-- print "/init"
end

-- required
function render()
	-- render simulation tree
	-- print "render"
	sim:render()
	-- sim:renderWithBloom()
	-- sim:renderWithBlur()
end

-- required
function update()
	print "update"
	sim:update()
	if sim:isKey( "quit" ) then sim:destroy() end

	r = earth:getRotation()
	r.Y = r.Y+0.01
	-- print ("r.Y:" .. r.Y )
	-- earth:setRotation( r )

	t = earth_atmosphere:getRotation()
	t.Y = t.Y - 0.01
	earth_atmosphere:setRotation( t )
	-- print ("r.Y:" .. r.Y )

	-- p = cam:getPosition()
end

-- end of file
